@@ -43,9 +43,10 @@ import org.slf4j.LoggerFactory;
 
 import org.apache.uniffle.client.api.ShuffleWriteClient;
 import org.apache.uniffle.client.response.SendShuffleDataResult;
-import org.apache.uniffle.common.RssShuffleUtils;
 import org.apache.uniffle.common.ShuffleBlockInfo;
 import org.apache.uniffle.common.ShuffleServerInfo;
+import org.apache.uniffle.common.compression.Codec;
+import org.apache.uniffle.common.config.RssConf;
 import org.apache.uniffle.common.exception.RssException;
 import org.apache.uniffle.common.util.ChecksumUtils;
 import org.apache.uniffle.common.util.ThreadUtils;
@@ -90,6 +91,8 @@ public class SortWriteBufferManager<K, V> {
   private long sortTime = 0;
   private final long maxBufferSize;
   private final ExecutorService sendExecutorService;
+  private final RssConf rssConf;
+  private final Codec codec;
 
   public SortWriteBufferManager(
       long maxMemSize,
@@ -114,7 +117,8 @@ public class SortWriteBufferManager<K, V> {
       boolean isMemoryShuffleEnabled,
       int sendThreadNum,
       double sendThreshold,
-      long maxBufferSize) {
+      long maxBufferSize,
+      RssConf rssConf) {
     this.maxMemSize = maxMemSize;
     this.taskAttemptId = taskAttemptId;
     this.batch = batch;
@@ -140,6 +144,8 @@ public class SortWriteBufferManager<K, V> {
     this.sendExecutorService  = Executors.newFixedThreadPool(
         sendThreadNum,
         ThreadUtils.getThreadFactory("send-thread-%d"));
+    this.rssConf = rssConf;
+    this.codec = Codec.newInstance(rssConf);
   }
 
   // todo: Single Buffer should also have its size limit
@@ -197,10 +203,7 @@ public class SortWriteBufferManager<K, V> {
         return o2.getDataLength() - o1.getDataLength();
       }
     });
-    int sendSize = batch;
-    if (batch > waitSendBuffers.size()) {
-      sendSize = waitSendBuffers.size();
-    }
+    int sendSize = Math.min(batch, waitSendBuffers.size());
     Iterator<SortWriteBuffer<K, V>> iterator = waitSendBuffers.iterator();
     int index = 0;
     List<ShuffleBlockInfo> shuffleBlocks = Lists.newArrayList();
@@ -234,7 +237,7 @@ public class SortWriteBufferManager<K, V> {
           for (ShuffleBlockInfo block : shuffleBlocks) {
             size += block.getFreeMemory();
           }
-          SendShuffleDataResult result = shuffleWriteClient.sendShuffleData(appId, shuffleBlocks);
+          SendShuffleDataResult result = shuffleWriteClient.sendShuffleData(appId, shuffleBlocks, () -> false);
           successBlockIds.addAll(result.getSuccessBlockIds());
           failedBlockIds.addAll(result.getFailedBlockIds());
         } catch (Throwable t) {
@@ -309,10 +312,10 @@ public class SortWriteBufferManager<K, V> {
     int partitionId = wb.getPartitionId();
     final int uncompressLength = data.length;
     long start = System.currentTimeMillis();
-    final byte[] compressed = RssShuffleUtils.compressData(data);
+    final byte[] compressed = codec.compress(data);
     final long crc32 = ChecksumUtils.getCrc32(compressed);
     compressTime += System.currentTimeMillis() - start;
-    final long blockId = RssMRUtils.getBlockId((long)partitionId, taskAttemptId, getNextSeqNo(partitionId));
+    final long blockId = RssMRUtils.getBlockId(partitionId, taskAttemptId, getNextSeqNo(partitionId));
     uncompressedDataLen += data.length;
     // add memory to indicate bytes which will be sent to shuffle server
     inSendListBytes.addAndGet(wb.getDataLength());
