@@ -46,10 +46,12 @@ import org.apache.uniffle.client.api.ShuffleReadClient;
 import org.apache.uniffle.client.factory.ShuffleClientFactory;
 import org.apache.uniffle.client.request.CreateShuffleReadClientRequest;
 import org.apache.uniffle.common.ShuffleServerInfo;
+import org.apache.uniffle.common.config.RssConf;
 
 public class RssShuffleReader<K, C> implements ShuffleReader<K, C> {
 
   private static final Logger LOG = LoggerFactory.getLogger(RssShuffleReader.class);
+  private final boolean expectedTaskIdsBitmapFilterEnable;
 
   private String appId;
   private int shuffleId;
@@ -69,6 +71,7 @@ public class RssShuffleReader<K, C> implements ShuffleReader<K, C> {
   private Roaring64NavigableMap taskIdBitmap;
   private List<ShuffleServerInfo> shuffleServerInfoList;
   private Configuration hadoopConf;
+  private RssConf rssConf;
 
   public RssShuffleReader(
       int startPartition,
@@ -83,7 +86,8 @@ public class RssShuffleReader<K, C> implements ShuffleReader<K, C> {
       int partitionNumPerRange,
       int partitionNum,
       Roaring64NavigableMap blockIdBitmap,
-      Roaring64NavigableMap taskIdBitmap) {
+      Roaring64NavigableMap taskIdBitmap,
+      RssConf rssConf) {
     this.appId = rssShuffleHandle.getAppId();
     this.startPartition = startPartition;
     this.endPartition = endPartition;
@@ -103,6 +107,8 @@ public class RssShuffleReader<K, C> implements ShuffleReader<K, C> {
     this.hadoopConf = hadoopConf;
     this.shuffleServerInfoList =
         (List<ShuffleServerInfo>) (rssShuffleHandle.getPartitionToServers().get(startPartition));
+    this.rssConf = rssConf;
+    expectedTaskIdsBitmapFilterEnable = shuffleServerInfoList.size() > 1;
   }
 
   @Override
@@ -111,11 +117,12 @@ public class RssShuffleReader<K, C> implements ShuffleReader<K, C> {
 
     CreateShuffleReadClientRequest request = new CreateShuffleReadClientRequest(
         appId, shuffleId, startPartition, storageType, basePath, indexReadLimit, readBufferSize,
-        partitionNumPerRange, partitionNum, blockIdBitmap, taskIdBitmap, shuffleServerInfoList, hadoopConf);
+        partitionNumPerRange, partitionNum, blockIdBitmap, taskIdBitmap,
+        shuffleServerInfoList, hadoopConf, expectedTaskIdsBitmapFilterEnable);
     ShuffleReadClient shuffleReadClient = ShuffleClientFactory.getInstance().createShuffleReadClient(request);
     RssShuffleDataIterator rssShuffleDataIterator = new RssShuffleDataIterator<K, C>(
         shuffleDependency.serializer(), shuffleReadClient,
-        new ReadMetrics(context.taskMetrics().createTempShuffleReadMetrics()));
+        new ReadMetrics(context.taskMetrics().createTempShuffleReadMetrics()), rssConf);
     CompletionIterator completionIterator =
         CompletionIterator$.MODULE$.apply(rssShuffleDataIterator, new AbstractFunction0<BoxedUnit>() {
           @Override
