@@ -17,22 +17,28 @@
 
 package org.apache.uniffle.client.impl;
 
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.Collections;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.Callable;
+import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.ForkJoinPool;
 import java.util.concurrent.Future;
 import java.util.concurrent.TimeUnit;
-import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
+import java.util.function.Supplier;
+import java.util.stream.Stream;
 
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
 import com.google.common.collect.Sets;
+import org.apache.commons.collections.CollectionUtils;
 import org.apache.hadoop.security.UserGroupInformation;
 import org.roaringbitmap.longlong.Roaring64NavigableMap;
 import org.slf4j.Logger;
@@ -44,6 +50,7 @@ import org.apache.uniffle.client.api.ShuffleWriteClient;
 import org.apache.uniffle.client.factory.CoordinatorClientFactory;
 import org.apache.uniffle.client.factory.ShuffleServerClientFactory;
 import org.apache.uniffle.client.request.RssAppHeartBeatRequest;
+import org.apache.uniffle.client.request.RssApplicationInfoRequest;
 import org.apache.uniffle.client.request.RssFetchClientConfRequest;
 import org.apache.uniffle.client.request.RssFetchRemoteStorageRequest;
 import org.apache.uniffle.client.request.RssFinishShuffleRequest;
@@ -54,9 +61,10 @@ import org.apache.uniffle.client.request.RssRegisterShuffleRequest;
 import org.apache.uniffle.client.request.RssReportShuffleResultRequest;
 import org.apache.uniffle.client.request.RssSendCommitRequest;
 import org.apache.uniffle.client.request.RssSendShuffleDataRequest;
+import org.apache.uniffle.client.request.RssUnregisterShuffleRequest;
 import org.apache.uniffle.client.response.ClientResponse;
-import org.apache.uniffle.client.response.ResponseStatusCode;
 import org.apache.uniffle.client.response.RssAppHeartBeatResponse;
+import org.apache.uniffle.client.response.RssApplicationInfoResponse;
 import org.apache.uniffle.client.response.RssFetchClientConfResponse;
 import org.apache.uniffle.client.response.RssFetchRemoteStorageResponse;
 import org.apache.uniffle.client.response.RssFinishShuffleResponse;
@@ -66,32 +74,41 @@ import org.apache.uniffle.client.response.RssRegisterShuffleResponse;
 import org.apache.uniffle.client.response.RssReportShuffleResultResponse;
 import org.apache.uniffle.client.response.RssSendCommitResponse;
 import org.apache.uniffle.client.response.RssSendShuffleDataResponse;
+import org.apache.uniffle.client.response.RssUnregisterShuffleResponse;
 import org.apache.uniffle.client.response.SendShuffleDataResult;
+import org.apache.uniffle.client.util.ClientUtils;
+import org.apache.uniffle.common.ClientType;
 import org.apache.uniffle.common.PartitionRange;
 import org.apache.uniffle.common.RemoteStorageInfo;
 import org.apache.uniffle.common.ShuffleAssignmentsInfo;
 import org.apache.uniffle.common.ShuffleBlockInfo;
+import org.apache.uniffle.common.ShuffleDataDistributionType;
 import org.apache.uniffle.common.ShuffleServerInfo;
 import org.apache.uniffle.common.exception.RssException;
+import org.apache.uniffle.common.rpc.StatusCode;
 import org.apache.uniffle.common.util.ThreadUtils;
 
 public class ShuffleWriteClientImpl implements ShuffleWriteClient {
 
   private static final Logger LOG = LoggerFactory.getLogger(ShuffleWriteClientImpl.class);
+
   private String clientType;
   private int retryMax;
   private long retryIntervalMax;
   private List<CoordinatorClient> coordinatorClients = Lists.newLinkedList();
-  private Set<ShuffleServerInfo> shuffleServerInfoSet = Sets.newConcurrentHashSet();
+  //appId -> shuffleId -> servers
+  private Map<String, Map<Integer, Set<ShuffleServerInfo>>> shuffleServerInfoMap = Maps.newConcurrentMap();
   private CoordinatorClientFactory coordinatorClientFactory;
   private ExecutorService heartBeatExecutorService;
   private int replica;
   private int replicaWrite;
   private int replicaRead;
   private boolean replicaSkipEnabled;
-  private int dataTranferPoolSize;
   private int dataCommitPoolSize = -1;
-  private final ForkJoinPool dataTransferPool;
+  private final ExecutorService dataTransferPool;
+  private final int unregisterThreadPoolSize;
+  private final int unregisterRequestTimeSec;
+  private Set<ShuffleServerInfo> defectiveServers;
 
   public ShuffleWriteClientImpl(
       String clientType,
@@ -102,32 +119,49 @@ public class ShuffleWriteClientImpl implements ShuffleWriteClient {
       int replicaWrite,
       int replicaRead,
       boolean replicaSkipEnabled,
-      int dataTranferPoolSize,
-      int dataCommitPoolSize) {
+      int dataTransferPoolSize,
+      int dataCommitPoolSize,
+      int unregisterThreadPoolSize,
+      int unregisterRequestTimeSec) {
     this.clientType = clientType;
     this.retryMax = retryMax;
     this.retryIntervalMax = retryIntervalMax;
-    this.coordinatorClientFactory = new CoordinatorClientFactory(clientType);
+    this.coordinatorClientFactory = new CoordinatorClientFactory(ClientType.valueOf(clientType));
     this.heartBeatExecutorService = Executors.newFixedThreadPool(heartBeatThreadNum,
         ThreadUtils.getThreadFactory("client-heartbeat-%d"));
     this.replica = replica;
     this.replicaWrite = replicaWrite;
     this.replicaRead = replicaRead;
     this.replicaSkipEnabled = replicaSkipEnabled;
-    this.dataTranferPoolSize = dataTranferPoolSize;
-    this.dataTransferPool = new ForkJoinPool(dataTranferPoolSize);
+    this.dataTransferPool = Executors.newFixedThreadPool(dataTransferPoolSize);
     this.dataCommitPoolSize = dataCommitPoolSize;
+    this.unregisterThreadPoolSize = unregisterThreadPoolSize;
+    this.unregisterRequestTimeSec = unregisterRequestTimeSec;
+    if (replica > 1) {
+      defectiveServers = Sets.newConcurrentHashSet();
+    }
   }
 
   private boolean sendShuffleDataAsync(
       String appId,
       Map<ShuffleServerInfo, Map<Integer, Map<Integer, List<ShuffleBlockInfo>>>> serverToBlocks,
       Map<ShuffleServerInfo, List<Long>> serverToBlockIds,
-      Map<Long, AtomicInteger> blockIdsTracker) {
+      Map<Long, AtomicInteger> blockIdsTracker, boolean allowFastFail,
+      Supplier<Boolean> needCancelRequest) {
+
+    if (serverToBlockIds == null) {
+      return true;
+    }
+
     // If one or more servers is failed, the sending is not totally successful.
-    AtomicBoolean isAllServersSuccess = new AtomicBoolean(true);
-    if (serverToBlocks != null) {
-      dataTransferPool.submit(() -> serverToBlocks.entrySet().parallelStream().forEach(entry -> {
+    List<CompletableFuture<Boolean>> futures = new ArrayList<>();
+    for (Map.Entry<ShuffleServerInfo, Map<Integer, Map<Integer, List<ShuffleBlockInfo>>>> entry :
+        serverToBlocks.entrySet()) {
+      CompletableFuture<Boolean> future = CompletableFuture.supplyAsync(() -> {
+        if (needCancelRequest.get()) {
+          LOG.info("The upstream task has been failed. Abort this data send.");
+          return true;
+        }
         ShuffleServerInfo ssi = entry.getKey();
         try {
           Map<Integer, Map<Integer, List<ShuffleBlockInfo>>> shuffleIdToBlocks = entry.getValue();
@@ -140,52 +174,83 @@ public class ShuffleWriteClientImpl implements ShuffleWriteClient {
           String logMsg = String.format("ShuffleWriteClientImpl sendShuffleData with %s blocks to %s cost: %s(ms)",
               serverToBlockIds.get(ssi).size(), ssi.getId(), System.currentTimeMillis() - s);
 
-          if (response.getStatusCode() == ResponseStatusCode.SUCCESS) {
+          if (response.getStatusCode() == StatusCode.SUCCESS) {
             // mark a replica of block that has been sent
             serverToBlockIds.get(ssi).forEach(block -> blockIdsTracker.get(block).incrementAndGet());
+            if (defectiveServers != null) {
+              defectiveServers.remove(ssi);
+            }
             LOG.info("{} successfully.", logMsg);
           } else {
-            isAllServersSuccess.set(false);
+            if (defectiveServers != null) {
+              defectiveServers.add(ssi);
+            }
             LOG.warn("{}, it failed wth statusCode[{}]", logMsg, response.getStatusCode());
+            return false;
           }
         } catch (Exception e) {
-          isAllServersSuccess.set(false);
+          if (defectiveServers != null) {
+            defectiveServers.add(ssi);
+          }
           LOG.warn("Send: " + serverToBlockIds.get(ssi).size() + " blocks to [" + ssi.getId() + "] failed.", e);
+          return false;
         }
-      })).join();
+        return true;
+      }, dataTransferPool);
+      futures.add(future);
     }
-    return isAllServersSuccess.get();
+
+    boolean result = ClientUtils.waitUntilDoneOrFail(futures, allowFastFail);
+    if (!result) {
+      LOG.error("Some shuffle data can't be sent to shuffle-server, is fast fail: {}, cancelled task size: {}",
+          allowFastFail, futures.size());
+    }
+    return result;
   }
 
-  private void genServerToBlocks(ShuffleBlockInfo sbi, List<ShuffleServerInfo> serverList,
+  void genServerToBlocks(
+      ShuffleBlockInfo sbi,
+      List<ShuffleServerInfo> serverList,
+      int replicaNum,
+      Collection<ShuffleServerInfo> excludeServers,
       Map<ShuffleServerInfo, Map<Integer, Map<Integer, List<ShuffleBlockInfo>>>> serverToBlocks,
-      Map<ShuffleServerInfo, List<Long>> serverToBlockIds) {
-    int partitionId = sbi.getPartitionId();
-    int shuffleId = sbi.getShuffleId();
-    for (ShuffleServerInfo ssi : serverList) {
-      if (!serverToBlockIds.containsKey(ssi)) {
-        serverToBlockIds.put(ssi, Lists.newArrayList());
-      }
-      serverToBlockIds.get(ssi).add(sbi.getBlockId());
-
-      if (!serverToBlocks.containsKey(ssi)) {
-        serverToBlocks.put(ssi, Maps.newHashMap());
-      }
-      Map<Integer, Map<Integer, List<ShuffleBlockInfo>>> shuffleIdToBlocks = serverToBlocks.get(ssi);
-      if (!shuffleIdToBlocks.containsKey(shuffleId)) {
-        shuffleIdToBlocks.put(shuffleId, Maps.newHashMap());
-      }
-
-      Map<Integer, List<ShuffleBlockInfo>> partitionToBlocks = shuffleIdToBlocks.get(shuffleId);
-      if (!partitionToBlocks.containsKey(partitionId)) {
-        partitionToBlocks.put(partitionId, Lists.newArrayList());
-      }
-      partitionToBlocks.get(partitionId).add(sbi);
+      Map<ShuffleServerInfo, List<Long>> serverToBlockIds,
+      boolean excludeDefectiveServers) {
+    if (replicaNum <= 0) {
+      return;
     }
+
+    Stream<ShuffleServerInfo> servers;
+    if (excludeDefectiveServers && CollectionUtils.isNotEmpty(defectiveServers)) {
+      servers = Stream.concat(serverList.stream().filter(x -> !defectiveServers.contains(x)),
+          serverList.stream().filter(defectiveServers::contains));
+    } else {
+      servers = serverList.stream();
+    }
+    if (excludeServers != null) {
+      servers = servers.filter(x -> !excludeServers.contains(x));
+    }
+
+    Stream<ShuffleServerInfo> selected = servers.limit(replicaNum);
+    if (excludeServers != null) {
+      selected = selected.peek(excludeServers::add);
+    }
+    selected.forEach(ssi -> {
+      serverToBlockIds.computeIfAbsent(ssi, id -> Lists.newArrayList())
+          .add(sbi.getBlockId());
+      serverToBlocks.computeIfAbsent(ssi, id -> Maps.newHashMap())
+          .computeIfAbsent(sbi.getShuffleId(), id -> Maps.newHashMap())
+          .computeIfAbsent(sbi.getPartitionId(), id -> Lists.newArrayList())
+          .add(sbi);
+    });
   }
 
+  /**
+   * The batch of sending belongs to the same task
+   */
   @Override
-  public SendShuffleDataResult sendShuffleData(String appId, List<ShuffleBlockInfo> shuffleBlockInfoList) {
+  public SendShuffleDataResult sendShuffleData(String appId, List<ShuffleBlockInfo> shuffleBlockInfoList,
+      Supplier<Boolean> needCancelRequest) {
 
     // shuffleServer -> shuffleId -> partitionId -> blocks
     Map<ShuffleServerInfo, Map<Integer,
@@ -208,19 +273,22 @@ public class ShuffleWriteClientImpl implements ShuffleWriteClient {
     for (ShuffleBlockInfo sbi : shuffleBlockInfoList) {
       List<ShuffleServerInfo> allServers = sbi.getShuffleServerInfos();
       if (replicaSkipEnabled) {
-        genServerToBlocks(sbi, allServers.subList(0, replicaWrite),
-            primaryServerToBlocks, primaryServerToBlockIds);
-        genServerToBlocks(sbi, allServers.subList(replicaWrite, replica),
-            secondaryServerToBlocks, secondaryServerToBlockIds);
+        Set<ShuffleServerInfo> excludeServers = Sets.newHashSet();
+        genServerToBlocks(sbi, allServers, replicaWrite, excludeServers,
+            primaryServerToBlocks, primaryServerToBlockIds, true);
+        genServerToBlocks(sbi, allServers,replica - replicaWrite,
+            excludeServers, secondaryServerToBlocks, secondaryServerToBlockIds, false);
       } else {
         // When replicaSkip is disabled, we send data to all replicas within one round.
-        genServerToBlocks(sbi, allServers,
-            primaryServerToBlocks, primaryServerToBlockIds);
+        genServerToBlocks(sbi, allServers, allServers.size(),
+            null, primaryServerToBlocks, primaryServerToBlockIds, false);
       }
     }
 
     // maintain the count of blocks that have been sent to the server
-    Map<Long, AtomicInteger> blockIdsTracker = Maps.newConcurrentMap();
+    // unnecessary to use concurrent hashmap here unless you need to insert or delete entries in other threads
+    // AtomicInteger is enough to reflect value changes in other threads
+    Map<Long, AtomicInteger> blockIdsTracker = Maps.newHashMap();
     primaryServerToBlockIds.values().forEach(
         blockList -> blockList.forEach(block -> blockIdsTracker.put(block, new AtomicInteger(0)))
     );
@@ -235,15 +303,28 @@ public class ShuffleWriteClientImpl implements ShuffleWriteClient {
 
     // sent the primary round of blocks.
     boolean isAllSuccess = sendShuffleDataAsync(
-        appId, primaryServerToBlocks, primaryServerToBlockIds, blockIdsTracker);
+        appId,
+        primaryServerToBlocks,
+        primaryServerToBlockIds,
+        blockIdsTracker,
+        secondaryServerToBlocks.isEmpty(),
+        needCancelRequest
+    );
 
     // The secondary round of blocks is sent only when the primary group issues failed sending.
     // This should be infrequent.
     // Even though the secondary round may send blocks more than replicaWrite replicas,
     // we do not apply complicated skipping logic, because server crash is rare in production environment.
-    if (!isAllSuccess && !secondaryServerToBlocks.isEmpty()) {
+    if (!isAllSuccess && !secondaryServerToBlocks.isEmpty() && !needCancelRequest.get()) {
       LOG.info("The sending of primary round is failed partially, so start the secondary round");
-      sendShuffleDataAsync(appId, secondaryServerToBlocks, secondaryServerToBlockIds, blockIdsTracker);
+      sendShuffleDataAsync(
+          appId,
+          secondaryServerToBlocks,
+          secondaryServerToBlockIds,
+          blockIdsTracker,
+          true,
+          needCancelRequest
+      );
     }
 
     // check success and failed blocks according to the replicaWrite
@@ -282,7 +363,7 @@ public class ShuffleWriteClientImpl implements ShuffleWriteClient {
           long startTime = System.currentTimeMillis();
           try {
             RssSendCommitResponse response = getShuffleServerClient(ssi).sendCommit(request);
-            if (response.getStatusCode() == ResponseStatusCode.SUCCESS) {
+            if (response.getStatusCode() == StatusCode.SUCCESS) {
               int commitCount = response.getCommitCount();
               LOG.info("Successfully sendCommit for appId[" + appId + "], shuffleId[" + shuffleId
                   + "] to ShuffleServer[" + ssi.getId() + "], cost "
@@ -291,7 +372,7 @@ public class ShuffleWriteClientImpl implements ShuffleWriteClient {
               if (commitCount >= numMaps) {
                 RssFinishShuffleResponse rfsResponse =
                     getShuffleServerClient(ssi).finishShuffle(new RssFinishShuffleRequest(appId, shuffleId));
-                if (rfsResponse.getStatusCode() != ResponseStatusCode.SUCCESS) {
+                if (rfsResponse.getStatusCode() != StatusCode.SUCCESS) {
                   String msg = "Failed to finish shuffle to " + ssi + " for shuffleId[" + shuffleId
                       + "] with statusCode " + rfsResponse.getStatusCode();
                   LOG.error(msg);
@@ -325,7 +406,8 @@ public class ShuffleWriteClientImpl implements ShuffleWriteClient {
       String appId,
       int shuffleId,
       List<PartitionRange> partitionRanges,
-      RemoteStorageInfo remoteStorage) {
+      RemoteStorageInfo remoteStorage,
+      ShuffleDataDistributionType dataDistributionType) {
     String user = null;
     try {
       user = UserGroupInformation.getCurrentUser().getShortUserName();
@@ -335,13 +417,13 @@ public class ShuffleWriteClientImpl implements ShuffleWriteClient {
     LOG.info("User: {}", user);
 
     RssRegisterShuffleRequest request =
-        new RssRegisterShuffleRequest(appId, shuffleId, partitionRanges, remoteStorage, user);
+        new RssRegisterShuffleRequest(appId, shuffleId, partitionRanges, remoteStorage, user, dataDistributionType);
     RssRegisterShuffleResponse response = getShuffleServerClient(shuffleServerInfo).registerShuffle(request);
 
     String msg = "Error happened when registerShuffle with appId[" + appId + "], shuffleId[" + shuffleId
         + "], " + shuffleServerInfo;
     throwExceptionIfNecessary(response, msg);
-    shuffleServerInfoSet.add(shuffleServerInfo);
+    addShuffleServer(appId, shuffleId, shuffleServerInfo);
   }
 
   @Override
@@ -353,10 +435,10 @@ public class ShuffleWriteClientImpl implements ShuffleWriteClient {
   @Override
   public Map<String, String> fetchClientConf(int timeoutMs) {
     RssFetchClientConfResponse response =
-        new RssFetchClientConfResponse(ResponseStatusCode.INTERNAL_ERROR, "Empty coordinator clients");
+        new RssFetchClientConfResponse(StatusCode.INTERNAL_ERROR, "Empty coordinator clients");
     for (CoordinatorClient coordinatorClient : coordinatorClients) {
       response = coordinatorClient.fetchClientConf(new RssFetchClientConfRequest(timeoutMs));
-      if (response.getStatusCode() == ResponseStatusCode.SUCCESS) {
+      if (response.getStatusCode() == StatusCode.SUCCESS) {
         LOG.info("Success to get conf from {}", coordinatorClient.getDesc());
         break;
       } else {
@@ -372,7 +454,7 @@ public class ShuffleWriteClientImpl implements ShuffleWriteClient {
     for (CoordinatorClient coordinatorClient : coordinatorClients) {
       RssFetchRemoteStorageResponse response =
           coordinatorClient.fetchRemoteStorage(new RssFetchRemoteStorageRequest(appId));
-      if (response.getStatusCode() == ResponseStatusCode.SUCCESS) {
+      if (response.getStatusCode() == StatusCode.SUCCESS) {
         remoteStorage = response.getRemoteStorageInfo();
         LOG.info("Success to get storage {} from {}", remoteStorage, coordinatorClient.getDesc());
         break;
@@ -385,11 +467,13 @@ public class ShuffleWriteClientImpl implements ShuffleWriteClient {
 
   @Override
   public ShuffleAssignmentsInfo getShuffleAssignments(String appId, int shuffleId, int partitionNum,
-      int partitionNumPerRange, Set<String> requiredTags, int assignmentShuffleServerNumber) {
+      int partitionNumPerRange, Set<String> requiredTags, int assignmentShuffleServerNumber,
+      int estimateTaskConcurrency) {
     RssGetShuffleAssignmentsRequest request = new RssGetShuffleAssignmentsRequest(
-        appId, shuffleId, partitionNum, partitionNumPerRange, replica, requiredTags, assignmentShuffleServerNumber);
+        appId, shuffleId, partitionNum, partitionNumPerRange, replica, requiredTags,
+        assignmentShuffleServerNumber, estimateTaskConcurrency);
 
-    RssGetShuffleAssignmentsResponse response = new RssGetShuffleAssignmentsResponse(ResponseStatusCode.INTERNAL_ERROR);
+    RssGetShuffleAssignmentsResponse response = new RssGetShuffleAssignmentsResponse(StatusCode.INTERNAL_ERROR);
     for (CoordinatorClient coordinatorClient : coordinatorClients) {
       try {
         response = coordinatorClient.getShuffleAssignments(request);
@@ -397,7 +481,7 @@ public class ShuffleWriteClientImpl implements ShuffleWriteClient {
         LOG.error(e.getMessage());
       }
 
-      if (response.getStatusCode() == ResponseStatusCode.SUCCESS) {
+      if (response.getStatusCode() == StatusCode.SUCCESS) {
         LOG.info("Success to get shuffle server assignment from {}", coordinatorClient.getDesc());
         break;
       }
@@ -418,31 +502,41 @@ public class ShuffleWriteClientImpl implements ShuffleWriteClient {
       long taskAttemptId,
       Map<Integer, List<Long>> partitionToBlockIds,
       int bitmapNum) {
-    Map<ShuffleServerInfo, List<Integer>> groupedPartitions = Maps.newConcurrentMap();
-    Map<Integer, Integer> partitionReportTracker = Maps.newConcurrentMap();
+    Map<ShuffleServerInfo, List<Integer>> groupedPartitions = Maps.newHashMap();
+    Map<Integer, Integer> partitionReportTracker = Maps.newHashMap();
     for (Map.Entry<Integer, List<ShuffleServerInfo>> entry : partitionToServers.entrySet()) {
+      int partitionIdx = entry.getKey();
       for (ShuffleServerInfo ssi : entry.getValue()) {
         if (!groupedPartitions.containsKey(ssi)) {
-          groupedPartitions.putIfAbsent(ssi, Lists.newArrayList());
+          groupedPartitions.put(ssi, Lists.newArrayList());
         }
-        groupedPartitions.get(ssi).add(entry.getKey());
+        groupedPartitions.get(ssi).add(partitionIdx);
+      }
+      if (CollectionUtils.isNotEmpty(partitionToBlockIds.get(partitionIdx))) {
+        partitionReportTracker.putIfAbsent(partitionIdx, 0);
       }
-      partitionReportTracker.putIfAbsent(entry.getKey(), 0);
     }
+
     for (Map.Entry<ShuffleServerInfo, List<Integer>> entry : groupedPartitions.entrySet()) {
       Map<Integer, List<Long>> requestBlockIds = Maps.newHashMap();
       for (Integer partitionId : entry.getValue()) {
-        requestBlockIds.put(partitionId, partitionToBlockIds.get(partitionId));
+        List<Long> blockIds = partitionToBlockIds.get(partitionId);
+        if (CollectionUtils.isNotEmpty(blockIds)) {
+          requestBlockIds.put(partitionId, blockIds);
+        }
+      }
+      if (requestBlockIds.isEmpty()) {
+        continue;
       }
       RssReportShuffleResultRequest request = new RssReportShuffleResultRequest(
           appId, shuffleId, taskAttemptId, requestBlockIds, bitmapNum);
       ShuffleServerInfo ssi = entry.getKey();
       try {
         RssReportShuffleResultResponse response = getShuffleServerClient(ssi).reportShuffleResult(request);
-        if (response.getStatusCode() == ResponseStatusCode.SUCCESS) {
+        if (response.getStatusCode() == StatusCode.SUCCESS) {
           LOG.info("Report shuffle result to " + ssi + " for appId[" + appId
               + "], shuffleId[" + shuffleId + "] successfully");
-          for (Integer partitionId : entry.getValue()) {
+          for (Integer partitionId : requestBlockIds.keySet()) {
             partitionReportTracker.put(partitionId, partitionReportTracker.get(partitionId) + 1);
           }
         } else {
@@ -474,7 +568,7 @@ public class ShuffleWriteClientImpl implements ShuffleWriteClient {
     for (ShuffleServerInfo ssi : shuffleServerInfoSet) {
       try {
         RssGetShuffleResultResponse response = getShuffleServerClient(ssi).getShuffleResult(request);
-        if (response.getStatusCode() == ResponseStatusCode.SUCCESS) {
+        if (response.getStatusCode() == StatusCode.SUCCESS) {
           // merge into blockIds from multiple servers.
           Roaring64NavigableMap blockIdBitmapOfServer = response.getBlockIdBitmap();
           blockIdBitmap.or(blockIdBitmapOfServer);
@@ -515,7 +609,7 @@ public class ShuffleWriteClientImpl implements ShuffleWriteClient {
       try {
         RssGetShuffleResultResponse response =
             getShuffleServerClient(shuffleServerInfo).getShuffleResultForMultiPart(request);
-        if (response.getStatusCode() == ResponseStatusCode.SUCCESS) {
+        if (response.getStatusCode() == StatusCode.SUCCESS) {
           // merge into blockIds from multiple servers.
           Roaring64NavigableMap blockIdBitmapOfServer = response.getBlockIdBitmap();
           blockIdBitmap.or(blockIdBitmapOfServer);
@@ -536,17 +630,49 @@ public class ShuffleWriteClientImpl implements ShuffleWriteClient {
     return blockIdBitmap;
   }
 
+  @Override
+  public void registerApplicationInfo(String appId, long timeoutMs, String user) {
+    RssApplicationInfoRequest request = new RssApplicationInfoRequest(appId, timeoutMs, user);
+    List<Callable<Void>> callableList = Lists.newArrayList();
+    coordinatorClients.forEach(coordinatorClient -> {
+      callableList.add(() -> {
+        try {
+          RssApplicationInfoResponse response = coordinatorClient.registerApplicationInfo(request);
+          if (response.getStatusCode() != StatusCode.SUCCESS) {
+            LOG.error("Failed to send applicationInfo to " + coordinatorClient.getDesc());
+          } else {
+            LOG.info("Successfully send applicationInfo to " + coordinatorClient.getDesc());
+          }
+        } catch (Exception e) {
+          LOG.warn("Error happened when send applicationInfo to " + coordinatorClient.getDesc(), e);
+        }
+        return null;
+      });
+    });
+    try {
+      List<Future<Void>> futures = heartBeatExecutorService.invokeAll(callableList, timeoutMs, TimeUnit.MILLISECONDS);
+      for (Future<Void> future : futures) {
+        if (!future.isDone()) {
+          future.cancel(true);
+        }
+      }
+    } catch (InterruptedException ie) {
+      LOG.warn("register application is interrupted", ie);
+    }
+  }
+
   @Override
   public void sendAppHeartbeat(String appId, long timeoutMs) {
     RssAppHeartBeatRequest request = new RssAppHeartBeatRequest(appId, timeoutMs);
     List<Callable<Void>> callableList = Lists.newArrayList();
-    shuffleServerInfoSet.stream().forEach(shuffleServerInfo -> {
+    Set<ShuffleServerInfo> allShuffleServers = getAllShuffleServers(appId);
+    allShuffleServers.forEach(shuffleServerInfo -> {
           callableList.add(() -> {
             try {
               ShuffleServerClient client =
                   ShuffleServerClientFactory.getInstance().getShuffleServerClient(clientType, shuffleServerInfo);
               RssAppHeartBeatResponse response = client.sendHeartBeat(request);
-              if (response.getStatusCode() != ResponseStatusCode.SUCCESS) {
+              if (response.getStatusCode() != StatusCode.SUCCESS) {
                 LOG.warn("Failed to send heartbeat to " + shuffleServerInfo);
               }
             } catch (Exception e) {
@@ -557,11 +683,11 @@ public class ShuffleWriteClientImpl implements ShuffleWriteClient {
         }
     );
 
-    coordinatorClients.stream().forEach(coordinatorClient -> {
+    coordinatorClients.forEach(coordinatorClient -> {
       callableList.add(() -> {
         try {
           RssAppHeartBeatResponse response = coordinatorClient.sendAppHeartBeat(request);
-          if (response.getStatusCode() != ResponseStatusCode.SUCCESS) {
+          if (response.getStatusCode() != StatusCode.SUCCESS) {
             LOG.warn("Failed to send heartbeat to " + coordinatorClient.getDesc());
           } else {
             LOG.info("Successfully send heartbeat to " + coordinatorClient.getDesc());
@@ -591,16 +717,106 @@ public class ShuffleWriteClientImpl implements ShuffleWriteClient {
     dataTransferPool.shutdownNow();
   }
 
+  @Override
+  public void unregisterShuffle(String appId, int shuffleId) {
+    RssUnregisterShuffleRequest request = new RssUnregisterShuffleRequest(appId, shuffleId);
+    List<Callable<Void>> callableList = Lists.newArrayList();
+
+    Map<Integer, Set<ShuffleServerInfo>> appServerMap = shuffleServerInfoMap.get(appId);
+    if (appServerMap == null) {
+      return;
+    }
+    Set<ShuffleServerInfo> shuffleServerInfos = appServerMap.get(shuffleId);
+    if (shuffleServerInfos == null) {
+      return;
+    }
+
+    shuffleServerInfos.forEach(shuffleServerInfo -> {
+          callableList.add(() -> {
+            try {
+              ShuffleServerClient client =
+                  ShuffleServerClientFactory.getInstance().getShuffleServerClient(clientType, shuffleServerInfo);
+              RssUnregisterShuffleResponse response = client.unregisterShuffle(request);
+              if (response.getStatusCode() != StatusCode.SUCCESS) {
+                LOG.warn("Failed to unregister shuffle to " + shuffleServerInfo);
+              }
+            } catch (Exception e) {
+              LOG.warn("Error happened when unregistering to " + shuffleServerInfo, e);
+            }
+            return null;
+          });
+        }
+    );
+
+    ExecutorService executorService = null;
+    try {
+      executorService =
+          Executors.newFixedThreadPool(
+              Math.min(unregisterThreadPoolSize, shuffleServerInfos.size()),
+              ThreadUtils.getThreadFactory("unregister-shuffle-%d")
+          );
+      List<Future<Void>> futures = executorService.invokeAll(callableList, unregisterRequestTimeSec, TimeUnit.SECONDS);
+      for (Future<Void> future : futures) {
+        if (!future.isDone()) {
+          future.cancel(true);
+        }
+      }
+    } catch (InterruptedException ie) {
+      LOG.warn("Unregister shuffle is interrupted", ie);
+    } finally {
+      if (executorService != null) {
+        executorService.shutdownNow();
+      }
+      removeShuffleServer(appId, shuffleId);
+    }
+  }
+
   private void throwExceptionIfNecessary(ClientResponse response, String errorMsg) {
-    if (response != null && response.getStatusCode() != ResponseStatusCode.SUCCESS) {
+    if (response != null && response.getStatusCode() != StatusCode.SUCCESS) {
       LOG.error(errorMsg);
       throw new RssException(errorMsg);
     }
   }
 
+  Set<ShuffleServerInfo> getAllShuffleServers(String appId) {
+    Map<Integer, Set<ShuffleServerInfo>> appServerMap = shuffleServerInfoMap.get(appId);
+    if (appServerMap == null) {
+      return Collections.EMPTY_SET;
+    }
+    Set<ShuffleServerInfo> serverInfos = Sets.newHashSet();
+    appServerMap.values().forEach(serverInfos::addAll);
+    return serverInfos;
+  }
+
   @VisibleForTesting
   public ShuffleServerClient getShuffleServerClient(ShuffleServerInfo shuffleServerInfo) {
     return ShuffleServerClientFactory.getInstance().getShuffleServerClient(clientType, shuffleServerInfo);
   }
 
+  @VisibleForTesting
+  Set<ShuffleServerInfo> getDefectiveServers() {
+    return defectiveServers;
+  }
+
+  void addShuffleServer(String appId, int shuffleId, ShuffleServerInfo serverInfo) {
+    Map<Integer, Set<ShuffleServerInfo>> appServerMap = shuffleServerInfoMap.get(appId);
+    if (appServerMap == null) {
+      appServerMap = Maps.newConcurrentMap();
+      shuffleServerInfoMap.put(appId, appServerMap);
+    }
+    Set<ShuffleServerInfo> shuffleServerInfos = appServerMap.get(shuffleId);
+    if (shuffleServerInfos == null) {
+      shuffleServerInfos = Sets.newConcurrentHashSet();
+      appServerMap.put(shuffleId, shuffleServerInfos);
+    }
+    shuffleServerInfos.add(serverInfo);
+  }
+
+  @VisibleForTesting
+  void removeShuffleServer(String appId, int shuffleId) {
+    Map<Integer, Set<ShuffleServerInfo>> appServerMap = shuffleServerInfoMap.get(appId);
+    if (appServerMap != null) {
+      appServerMap.remove(shuffleId);
+    }
+  }
 }
