@@ -17,11 +17,10 @@
 
 package org.apache.uniffle.common.util;
 
-import java.lang.reflect.Field;
 import java.net.Inet4Address;
 import java.net.InetAddress;
 import java.net.NetworkInterface;
-import java.nio.ByteBuffer;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.List;
 import java.util.Map;
@@ -35,16 +34,18 @@ import com.google.common.collect.Sets;
 import org.junit.jupiter.api.Test;
 import org.roaringbitmap.longlong.Roaring64NavigableMap;
 
-import org.apache.uniffle.common.BufferSegment;
-import org.apache.uniffle.common.ShuffleDataSegment;
-import org.apache.uniffle.common.ShuffleIndexResult;
 import org.apache.uniffle.common.ShuffleServerInfo;
+import org.apache.uniffle.common.config.RssBaseConf;
+import org.apache.uniffle.common.config.RssConf;
 
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
 import static org.junit.jupiter.api.Assertions.assertNotSame;
+import static org.junit.jupiter.api.Assertions.assertNull;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 import static org.junit.jupiter.api.Assertions.fail;
+import static uk.org.webcompere.systemstubs.SystemStubs.withEnvironmentVariable;
 
 public class RssUtilsTest {
 
@@ -67,20 +68,20 @@ public class RssUtilsTest {
       InetAddress ia = InetAddress.getByName(realIp);
       assertTrue(ia instanceof Inet4Address);
       assertFalse(ia.isLinkLocalAddress() || ia.isAnyLocalAddress() || ia.isLoopbackAddress());
-      assertTrue(NetworkInterface.getByInetAddress(ia) != null);
+      assertNotNull(NetworkInterface.getByInetAddress(ia));
       assertTrue(ia.isReachable(5000));
-      setEnv("RSS_IP", "8.8.8.8");
-      assertEquals("8.8.8.8", RssUtils.getHostIp());
-      setEnv("RSS_IP", "xxxx");
-      boolean isException = false;
-      try {
-        RssUtils.getHostIp();
-      } catch (Exception e) {
-        isException = true;
-      }
-      setEnv("RSS_IP", realIp);
-      RssUtils.getHostIp();
-      assertTrue(isException);
+      withEnvironmentVariable("RSS_IP", "8.8.8.8")
+          .execute(() -> assertEquals("8.8.8.8", RssUtils.getHostIp()));
+      withEnvironmentVariable("RSS_IP", "xxxx").execute(() -> {
+        boolean isException = false;
+        try {
+          RssUtils.getHostIp();
+        } catch (Exception e) {
+          isException = true;
+        }
+        assertTrue(isException);
+      });
+      withEnvironmentVariable("RSS_IP", realIp).execute(RssUtils::getHostIp);
     } catch (Exception e) {
       fail(e.getMessage());
     }
@@ -103,69 +104,6 @@ public class RssUtilsTest {
     assertEquals(bitmap1, bitmap2);
   }
 
-  @Test
-  public void testShuffleIndexSegment() {
-    ShuffleIndexResult shuffleIndexResult = new ShuffleIndexResult();
-    List<ShuffleDataSegment> shuffleDataSegments =
-        RssUtils.transIndexDataToSegments(shuffleIndexResult, 1000);
-    assertTrue(shuffleDataSegments.isEmpty());
-
-    int readBufferSize = 32;
-    int totalLength = 0;
-    List<BufferSegment> bufferSegments = Lists.newArrayList();
-    int[] dataSegmentLength = new int[]{32, 16, 10, 32, 6};
-
-    for (int i = 0; i < dataSegmentLength.length; ++i) {
-      long offset = totalLength;
-      int length = dataSegmentLength[i];
-      bufferSegments.add(new BufferSegment(i, offset, length, i, i, i));
-      totalLength += length;
-    }
-
-    // those 5 segment's data length are [32, 16, 10, 32, 6] so the index should be
-    // split into 3 ShuffleDataSegment, which are [32, 16 + 10 + 32, 6]
-    int expectedTotalSegmentNum = 3;
-    ByteBuffer byteBuffer = ByteBuffer.allocate(5 * 40);
-
-    for (BufferSegment bufferSegment : bufferSegments) {
-      byteBuffer.putLong(bufferSegment.getOffset());
-      byteBuffer.putInt(bufferSegment.getLength());
-      byteBuffer.putInt(bufferSegment.getUncompressLength());
-      byteBuffer.putLong(bufferSegment.getCrc());
-      byteBuffer.putLong(bufferSegment.getBlockId());
-      byteBuffer.putLong(bufferSegment.getTaskAttemptId());
-    }
-
-    byte[] data = byteBuffer.array();
-    shuffleDataSegments = RssUtils.transIndexDataToSegments(new ShuffleIndexResult(data, -1), readBufferSize);
-    assertEquals(expectedTotalSegmentNum, shuffleDataSegments.size());
-
-    assertEquals(0, shuffleDataSegments.get(0).getOffset());
-    assertEquals(32, shuffleDataSegments.get(0).getLength());
-    assertEquals(1, shuffleDataSegments.get(0).getBufferSegments().size());
-
-    assertEquals(32, shuffleDataSegments.get(1).getOffset());
-    assertEquals(58, shuffleDataSegments.get(1).getLength());
-    assertEquals(3,shuffleDataSegments.get(1).getBufferSegments().size());
-
-    assertEquals(90, shuffleDataSegments.get(2).getOffset());
-    assertEquals(6, shuffleDataSegments.get(2).getLength());
-    assertEquals(1, shuffleDataSegments.get(2).getBufferSegments().size());
-
-    ByteBuffer incompleteByteBuffer = ByteBuffer.allocate(12);
-    incompleteByteBuffer.putLong(1L);
-    incompleteByteBuffer.putInt(2);
-    data = incompleteByteBuffer.array();
-    // It should throw exception
-    try {
-      RssUtils.transIndexDataToSegments(new ShuffleIndexResult(data, -1), readBufferSize);
-      fail();
-    } catch (Exception e) {
-      // ignore
-      assertTrue(e.getMessage().contains("Read index data under flow"));
-    }
-  }
-
   @Test
   public void getMetricNameForHostNameTest() {
     assertEquals("a_b_c", RssUtils.getMetricNameForHostName("a.b.c"));
@@ -243,6 +181,24 @@ public class RssUtilsTest {
     assertEquals(serverToPartitions.get(server4), Sets.newHashSet(2, 4));
   }
 
+  @Test
+  public void testGetConfiguredLocalDirs() throws Exception {
+    RssConf conf = new RssConf();
+    withEnvironmentVariable(RssUtils.RSS_LOCAL_DIR_KEY, "/path/a").execute(() -> {
+      assertEquals(Collections.singletonList("/path/a"), RssUtils.getConfiguredLocalDirs(conf));
+    });
+
+    withEnvironmentVariable(RssUtils.RSS_LOCAL_DIR_KEY, "/path/a,/path/b").execute(() -> {
+      assertEquals(Arrays.asList("/path/a", "/path/b"), RssUtils.getConfiguredLocalDirs(conf));
+    });
+
+    withEnvironmentVariable(RssUtils.RSS_LOCAL_DIR_KEY, null).execute(() -> {
+      assertNull(RssUtils.getConfiguredLocalDirs(conf));
+      conf.set(RssBaseConf.RSS_STORAGE_BASE_PATH, Arrays.asList("/path/a", "/path/b"));
+      assertEquals(Arrays.asList("/path/a", "/path/b"), RssUtils.getConfiguredLocalDirs(conf));
+    });
+  }
+
   // Copy from ClientUtils
   private Long getBlockId(long partitionId, long taskAttemptId, long atomicInt) {
     return (atomicInt << (Constants.PARTITION_ID_MAX_LENGTH + Constants.TASK_ATTEMPT_ID_MAX_LENGTH))
@@ -267,20 +223,6 @@ public class RssUtilsTest {
     }
   }
 
-  @SuppressWarnings("unchecked")
-  public static void setEnv(String key, String value) {
-    try {
-      Map<String, String> env = System.getenv();
-      Class<?> cl = env.getClass();
-      Field field = cl.getDeclaredField("m");
-      field.setAccessible(true);
-      Map<String, String> writableEnv = (Map<String, String>) field.get(env);
-      writableEnv.put(key, value);
-    } catch (Exception e) {
-      throw new IllegalStateException("Failed to set environment variable", e);
-    }
-  }
-
   public static class RssUtilTestDummySuccess implements RssUtilTestDummy {
     private final String s;
 
