@@ -18,13 +18,11 @@
 package org.apache.uniffle.coordinator;
 
 import java.util.List;
-import java.util.Map;
 
 import org.apache.uniffle.common.config.ConfigOption;
 import org.apache.uniffle.common.config.ConfigOptions;
 import org.apache.uniffle.common.config.ConfigUtils;
 import org.apache.uniffle.common.config.RssBaseConf;
-import org.apache.uniffle.common.util.RssUtils;
 import org.apache.uniffle.coordinator.strategy.assignment.AbstractAssignmentStrategy;
 import org.apache.uniffle.coordinator.strategy.assignment.AssignmentStrategyFactory;
 
@@ -209,22 +207,6 @@ public class CoordinatorConf extends RssBaseConf {
   }
 
   public boolean loadConfFromFile(String fileName) {
-    Map<String, String> properties = RssUtils.getPropertiesFromFile(fileName);
-
-    if (properties == null) {
-      return false;
-    }
-
-    loadCommonConf(properties);
-
-    List<ConfigOption<Object>> configOptions = ConfigUtils.getAllConfigOptions(CoordinatorConf.class);
-    properties.forEach((k, v) -> {
-      configOptions.forEach(config -> {
-        if (config.key().equalsIgnoreCase(k)) {
-          set(config, ConfigUtils.convertValue(v, config.getClazz()));
-        }
-      });
-    });
-    return true;
+    return loadConfFromFile(fileName, ConfigUtils.getAllConfigOptions(CoordinatorConf.class));
   }
 }
