@@ -44,8 +44,10 @@ import org.apache.hadoop.fs.Path;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import org.apache.uniffle.common.config.RssConf;
 import org.apache.uniffle.common.filesystem.HadoopFilesystemProvider;
 import org.apache.uniffle.common.util.ThreadUtils;
+import org.apache.uniffle.coordinator.metric.CoordinatorMetrics;
 
 public class SimpleClusterManager implements ClusterManager {
 
@@ -57,7 +59,7 @@ public class SimpleClusterManager implements ClusterManager {
   private Map<String, Set<ServerNode>> tagToNodes = Maps.newConcurrentMap();
   private AtomicLong excludeLastModify = new AtomicLong(0L);
   private long heartbeatTimeout;
-  private int shuffleNodesMax;
+  private volatile int shuffleNodesMax;
   private ScheduledExecutorService scheduledExecutorService;
   private ScheduledExecutorService checkNodesExecutorService;
   private FileSystem hadoopFileSystem;
@@ -65,6 +67,11 @@ public class SimpleClusterManager implements ClusterManager {
   private long outputAliveServerCount = 0;
   private final long periodicOutputIntervalTimes;
 
+  private long startTime;
+  private boolean startupSilentPeriodEnabled;
+  private long startupSilentPeriodDurationMs;
+  private boolean readyForServe = false;
+
   public SimpleClusterManager(CoordinatorConf conf, Configuration hadoopConf) throws Exception {
     this.shuffleNodesMax = conf.getInteger(CoordinatorConf.COORDINATOR_SHUFFLE_NODES_MAX);
     this.heartbeatTimeout = conf.getLong(CoordinatorConf.COORDINATOR_HEARTBEAT_TIMEOUT);
@@ -72,9 +79,12 @@ public class SimpleClusterManager implements ClusterManager {
     scheduledExecutorService = Executors.newSingleThreadScheduledExecutor(
         ThreadUtils.getThreadFactory("SimpleClusterManager-%d"));
 
+    this.startupSilentPeriodEnabled = conf.get(CoordinatorConf.COORDINATOR_START_SILENT_PERIOD_ENABLED);
+    this.startupSilentPeriodDurationMs = conf.get(CoordinatorConf.COORDINATOR_START_SILENT_PERIOD_DURATION);
+
     periodicOutputIntervalTimes = conf.get(CoordinatorConf.COORDINATOR_NODES_PERIODIC_OUTPUT_INTERVAL_TIMES);
     scheduledExecutorService.scheduleAtFixedRate(
-        () -> nodesCheck(), heartbeatTimeout / 3,
+        this::nodesCheck, heartbeatTimeout / 3,
         heartbeatTimeout / 3, TimeUnit.MILLISECONDS);
 
     String excludeNodesPath = conf.getString(CoordinatorConf.COORDINATOR_EXCLUDE_NODES_FILE_PATH, "");
@@ -86,6 +96,8 @@ public class SimpleClusterManager implements ClusterManager {
       checkNodesExecutorService.scheduleAtFixedRate(
           () -> updateExcludeNodes(excludeNodesPath), updateNodesInterval, updateNodesInterval, TimeUnit.MILLISECONDS);
     }
+
+    this.startTime = System.currentTimeMillis();
   }
 
   void nodesCheck() {
@@ -216,7 +228,7 @@ public class SimpleClusterManager implements ClusterManager {
   }
 
   @VisibleForTesting
-  void clear() {
+  public void clear() {
     servers.clear();
   }
 
@@ -225,6 +237,19 @@ public class SimpleClusterManager implements ClusterManager {
     return shuffleNodesMax;
   }
 
+  @Override
+  public boolean isReadyForServe() {
+    if (!startupSilentPeriodEnabled) {
+      return true;
+    }
+
+    if (!readyForServe && System.currentTimeMillis() - startTime > startupSilentPeriodDurationMs) {
+      readyForServe = true;
+    }
+
+    return readyForServe;
+  }
+
   @Override
   public void close() throws IOException {
     if (hadoopFileSystem != null) {
@@ -239,4 +264,33 @@ public class SimpleClusterManager implements ClusterManager {
       checkNodesExecutorService.shutdown();
     }
   }
+
+  @VisibleForTesting
+  public void setStartTime(long startTime) {
+    this.startTime = startTime;
+  }
+
+  @VisibleForTesting
+  public void setReadyForServe(boolean readyForServe) {
+    this.readyForServe = readyForServe;
+  }
+
+  @VisibleForTesting
+  public void setStartupSilentPeriodEnabled(boolean startupSilentPeriodEnabled) {
+    this.startupSilentPeriodEnabled = startupSilentPeriodEnabled;
+  }
+
+  @Override
+  public void reconfigure(RssConf conf) {
+    int nodeMax = conf.getInteger(CoordinatorConf.COORDINATOR_SHUFFLE_NODES_MAX);
+    if (nodeMax != shuffleNodesMax) {
+      LOG.warn("Coordinator update new shuffleNodesMax " + nodeMax);
+      shuffleNodesMax = nodeMax;
+    }
+  }
+
+  @Override
+  public boolean isPropertyReconfigurable(String property) {
+    return CoordinatorConf.COORDINATOR_SHUFFLE_NODES_MAX.key().equals(property);
+  }
 }
