@@ -19,6 +19,7 @@ package org.apache.uniffle.test;
 
 import java.io.OutputStreamWriter;
 import java.io.PrintWriter;
+import java.util.Collections;
 import java.util.Properties;
 import java.util.concurrent.TimeUnit;
 
@@ -67,6 +68,7 @@ public class GetReaderTest extends IntegrationTestBase {
   public void test() throws Exception {
     SparkConf sparkConf = new SparkConf();
     sparkConf.set("spark.shuffle.manager", "org.apache.spark.shuffle.RssShuffleManager");
+    sparkConf.set("spark.serializer", "org.apache.spark.serializer.KryoSerializer");
     sparkConf.set(RssSparkConfig.RSS_COORDINATOR_QUORUM.key(), COORDINATOR_QUORUM);
     sparkConf.setMaster("local[4]");
     final String remoteStorage1 = "hdfs://h1/p1";
@@ -89,6 +91,8 @@ public class GetReaderTest extends IntegrationTestBase {
     coordinatorConf.setInteger("rss.coordinator.dynamicClientConf.updateIntervalSec", 1);
     coordinatorConf.setInteger("rss.coordinator.access.candidates.updateIntervalSec", 1);
     coordinatorConf.setInteger("rss.coordinator.access.loadChecker.serverNum.threshold", 1);
+    coordinatorConf.setLong("rss.coordinator.remote.storage.schedule.time", 200);
+    coordinatorConf.setInteger("rss.coordinator.remote.storage.schedule.access.times", 1);
     createCoordinatorServer(coordinatorConf);
 
     ShuffleServerConf shuffleServerConf = getShuffleServerConf();
@@ -105,6 +109,15 @@ public class GetReaderTest extends IntegrationTestBase {
     assertEquals(remoteStorage1, remoteStorageInfo1.getPath());
     assertTrue(remoteStorageInfo1.getConfItems().isEmpty());
 
+    // emptyRDD case
+    JavaPairRDD<String, Tuple2<Integer, Integer>> javaEmptyPairRDD1 = TestUtils.combineByKeyRDD(
+        TestUtils.getEmptyRDD(jsc1));
+    ShuffleDependency emptyShuffleDependency1 = (ShuffleDependency) javaEmptyPairRDD1.rdd().dependencies().head();
+    RssShuffleHandle emptyRssShuffleHandle1 = (RssShuffleHandle) emptyShuffleDependency1.shuffleHandle();
+    assertEquals(javaEmptyPairRDD1.rdd().dependencies().head().rdd().getNumPartitions(), 0);
+    assertEquals(emptyRssShuffleHandle1.getPartitionToServers(), Collections.emptyMap());
+    assertEquals(emptyRssShuffleHandle1.getRemoteStorage(),RemoteStorageInfo.EMPTY_REMOTE_STORAGE);
+
     // the same app would get the same storage info
     JavaPairRDD<String, Tuple2<Integer, Integer>> javaPairRDD2 = TestUtils.combineByKeyRDD(TestUtils.getRDD(jsc1));
     ShuffleDependency shuffleDependency2 = (ShuffleDependency) javaPairRDD2.rdd().dependencies().head();
@@ -130,6 +143,8 @@ public class GetReaderTest extends IntegrationTestBase {
     JavaPairRDD<String, Tuple2<Integer, Integer>> javaPairRDD = TestUtils.combineByKeyRDD(TestUtils.getRDD(jsc2));
     ShuffleDependency shuffleDependency = (ShuffleDependency) javaPairRDD.rdd().dependencies().head();
     rssShuffleHandle = (RssShuffleHandle) shuffleDependency.shuffleHandle();
+    // the reason for sleep here is to ensure that threads can be scheduled normally
+    Thread.sleep(500);
     RemoteStorageInfo remoteStorageInfo3 = rssShuffleHandle.getRemoteStorage();
     assertEquals(remoteStorage1, remoteStorageInfo1.getPath());
     assertEquals(2, remoteStorageInfo3.getConfItems().size());
@@ -155,6 +170,8 @@ public class GetReaderTest extends IntegrationTestBase {
     commonHadoopConf = jsc2.hadoopConfiguration();
     assertNull(commonHadoopConf.get("k1"));
     assertNull(commonHadoopConf.get("k2"));
+
+    sparkSession.close();
   }
 
   private static class MockTaskContext extends TaskContext {
@@ -248,6 +265,10 @@ public class GetReaderTest extends IntegrationTestBase {
       return null;
     }
 
+    public int cpus() {
+      return Runtime.getRuntime().availableProcessors();
+    }
+
     @Override
     public java.util.Map<String, ResourceInformation> resourcesJMap() {
       return null;
