@@ -17,6 +17,8 @@
 
 package org.apache.uniffle.client.impl.grpc;
 
+import java.util.ArrayList;
+import java.util.Collections;
 import java.util.List;
 import java.util.Map;
 import java.util.Random;
@@ -25,6 +27,7 @@ import java.util.concurrent.TimeUnit;
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.collect.Lists;
 import com.google.protobuf.ByteString;
+import com.google.protobuf.UnsafeByteOperations;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -40,7 +43,7 @@ import org.apache.uniffle.client.request.RssRegisterShuffleRequest;
 import org.apache.uniffle.client.request.RssReportShuffleResultRequest;
 import org.apache.uniffle.client.request.RssSendCommitRequest;
 import org.apache.uniffle.client.request.RssSendShuffleDataRequest;
-import org.apache.uniffle.client.response.ResponseStatusCode;
+import org.apache.uniffle.client.request.RssUnregisterShuffleRequest;
 import org.apache.uniffle.client.response.RssAppHeartBeatResponse;
 import org.apache.uniffle.client.response.RssFinishShuffleResponse;
 import org.apache.uniffle.client.response.RssGetInMemoryShuffleDataResponse;
@@ -51,13 +54,18 @@ import org.apache.uniffle.client.response.RssRegisterShuffleResponse;
 import org.apache.uniffle.client.response.RssReportShuffleResultResponse;
 import org.apache.uniffle.client.response.RssSendCommitResponse;
 import org.apache.uniffle.client.response.RssSendShuffleDataResponse;
+import org.apache.uniffle.client.response.RssUnregisterShuffleResponse;
 import org.apache.uniffle.common.BufferSegment;
 import org.apache.uniffle.common.PartitionRange;
 import org.apache.uniffle.common.RemoteStorageInfo;
 import org.apache.uniffle.common.ShuffleBlockInfo;
+import org.apache.uniffle.common.ShuffleDataDistributionType;
 import org.apache.uniffle.common.exception.NotRetryException;
 import org.apache.uniffle.common.exception.RssException;
+import org.apache.uniffle.common.rpc.StatusCode;
 import org.apache.uniffle.common.util.RetryUtils;
+import org.apache.uniffle.common.util.RssUtils;
+import org.apache.uniffle.proto.RssProtos;
 import org.apache.uniffle.proto.RssProtos.AppHeartBeatRequest;
 import org.apache.uniffle.proto.RssProtos.AppHeartBeatResponse;
 import org.apache.uniffle.proto.RssProtos.FinishShuffleRequest;
@@ -89,7 +97,6 @@ import org.apache.uniffle.proto.RssProtos.ShuffleDataBlockSegment;
 import org.apache.uniffle.proto.RssProtos.ShufflePartitionRange;
 import org.apache.uniffle.proto.RssProtos.ShuffleRegisterRequest;
 import org.apache.uniffle.proto.RssProtos.ShuffleRegisterResponse;
-import org.apache.uniffle.proto.RssProtos.StatusCode;
 import org.apache.uniffle.proto.ShuffleServerGrpc;
 import org.apache.uniffle.proto.ShuffleServerGrpc.ShuffleServerBlockingStub;
 
@@ -113,7 +120,7 @@ public class ShuffleServerGrpcClient extends GrpcClient implements ShuffleServer
     super(host, port, maxRetryAttempts, usePlaintext);
     blockingStub = ShuffleServerGrpc.newBlockingStub(channel);
   }
-  
+
   public ShuffleServerBlockingStub getBlockingStub() {
     return blockingStub.withDeadlineAfter(rpcTimeout, TimeUnit.MILLISECONDS);
   }
@@ -128,12 +135,14 @@ public class ShuffleServerGrpcClient extends GrpcClient implements ShuffleServer
       int shuffleId,
       List<PartitionRange> partitionRanges,
       RemoteStorageInfo remoteStorageInfo,
-      String user) {
+      String user,
+      ShuffleDataDistributionType dataDistributionType) {
     ShuffleRegisterRequest.Builder reqBuilder = ShuffleRegisterRequest.newBuilder();
     reqBuilder
         .setAppId(appId)
         .setShuffleId(shuffleId)
         .setUser(user)
+        .setShuffleDataDistribution(RssProtos.DataDistribution.valueOf(dataDistributionType.name()))
         .addAllPartitionRanges(toShufflePartitionRanges(partitionRanges));
     RemoteStorage.Builder rsBuilder = RemoteStorage.newBuilder();
     rsBuilder.setPath(remoteStorageInfo.getPath());
@@ -171,26 +180,51 @@ public class ShuffleServerGrpcClient extends GrpcClient implements ShuffleServer
     return blockingStub.withDeadlineAfter(timeout, TimeUnit.MILLISECONDS).appHeartbeat(request);
   }
 
-  public long requirePreAllocation(int requireSize, int retryMax, long retryIntervalMax) {
-    RequireBufferRequest rpcRequest = RequireBufferRequest.newBuilder().setRequireSize(requireSize).build();
+  // Only for tests
+  @VisibleForTesting
+  public long requirePreAllocation(int requireSize, int retryMax, long retryIntervalMax) throws Exception {
+    return requirePreAllocation(
+        "EMPTY",
+        0,
+        Collections.emptyList(),
+        requireSize,
+        retryMax,
+        retryIntervalMax
+    );
+  }
+
+  public long requirePreAllocation(
+      String appId,
+      int shuffleId,
+      List<Integer> partitionIds,
+      int requireSize,
+      int retryMax,
+      long retryIntervalMax) {
+    RequireBufferRequest rpcRequest = RequireBufferRequest.newBuilder()
+        .setShuffleId(shuffleId)
+        .addAllPartitionIds(partitionIds)
+        .setAppId(appId)
+        .setRequireSize(requireSize)
+        .build();
+
     long start = System.currentTimeMillis();
     RequireBufferResponse rpcResponse = getBlockingStub().requireBuffer(rpcRequest);
     int retry = 0;
     long result = FAILED_REQUIRE_ID;
     Random random = new Random();
     final int backOffBase = 2000;
-    while (rpcResponse.getStatus() == StatusCode.NO_BUFFER) {
+    while (rpcResponse.getStatus() == RssProtos.StatusCode.NO_BUFFER) {
       LOG.info("Can't require " + requireSize + " bytes from " + host + ":" + port + ", sleep and try["
           + retry + "] again");
       if (retry >= retryMax) {
         LOG.warn("ShuffleServer " + host + ":" + port + " is full and can't send shuffle"
-            + " data successfully after retry " + retryMax + " times, cost: {}(ms)",
+                + " data successfully after retry " + retryMax + " times, cost: {}(ms)",
             System.currentTimeMillis() - start);
         return result;
       }
       try {
         long backoffTime =
-            Math.min(retryIntervalMax, backOffBase * (1 << Math.min(retry, 16)) + random.nextInt(backOffBase));
+            Math.min(retryIntervalMax, backOffBase * (1L << Math.min(retry, 16)) + random.nextInt(backOffBase));
         Thread.sleep(backoffTime);
       } catch (Exception e) {
         LOG.warn("Exception happened when require pre allocation from " + host + ":" + port, e);
@@ -198,7 +232,7 @@ public class ShuffleServerGrpcClient extends GrpcClient implements ShuffleServer
       rpcResponse = getBlockingStub().requireBuffer(rpcRequest);
       retry++;
     }
-    if (rpcResponse.getStatus() == StatusCode.SUCCESS) {
+    if (rpcResponse.getStatus() == RssProtos.StatusCode.SUCCESS) {
       LOG.info("Require preAllocated size of {} from {}:{}, cost: {}(ms)",
           requireSize, host, port, System.currentTimeMillis() - start);
       result = rpcResponse.getRequireBufferId();
@@ -206,6 +240,35 @@ public class ShuffleServerGrpcClient extends GrpcClient implements ShuffleServer
     return result;
   }
 
+  private RssProtos.ShuffleUnregisterResponse doUnregisterShuffle(String appId, int shuffleId) {
+    RssProtos.ShuffleUnregisterRequest request = RssProtos.ShuffleUnregisterRequest.newBuilder()
+        .setAppId(appId)
+        .setShuffleId(shuffleId)
+        .build();
+    return blockingStub.unregisterShuffle(request);
+  }
+
+  @Override
+  public RssUnregisterShuffleResponse unregisterShuffle(RssUnregisterShuffleRequest request) {
+    RssProtos.ShuffleUnregisterResponse rpcResponse = doUnregisterShuffle(request.getAppId(), request.getShuffleId());
+
+    RssUnregisterShuffleResponse response;
+    RssProtos.StatusCode statusCode = rpcResponse.getStatus();
+
+    switch (statusCode) {
+      case SUCCESS:
+        response = new RssUnregisterShuffleResponse(StatusCode.SUCCESS);
+        break;
+      default:
+        String msg = String.format("Errors on unregister shuffle to %s:%s for appId[%s].shuffleId[%], error: %s",
+            host, port, request.getAppId(), request.getShuffleId(), rpcResponse.getRetMsg());
+        LOG.error(msg);
+        throw new RssException(msg);
+    }
+
+    return response;
+  }
+
   @Override
   public RssRegisterShuffleResponse registerShuffle(RssRegisterShuffleRequest request) {
     ShuffleRegisterResponse rpcResponse = doRegisterShuffle(
@@ -213,14 +276,15 @@ public class ShuffleServerGrpcClient extends GrpcClient implements ShuffleServer
         request.getShuffleId(),
         request.getPartitionRanges(),
         request.getRemoteStorageInfo(),
-        request.getUser()
+        request.getUser(),
+        request.getDataDistributionType()
     );
 
     RssRegisterShuffleResponse response;
-    StatusCode statusCode = rpcResponse.getStatus();
+    RssProtos.StatusCode statusCode = rpcResponse.getStatus();
     switch (statusCode) {
       case SUCCESS:
-        response = new RssRegisterShuffleResponse(ResponseStatusCode.SUCCESS);
+        response = new RssRegisterShuffleResponse(StatusCode.SUCCESS);
         break;
       default:
         String msg = "Can't register shuffle to " + host + ":" + port
@@ -244,6 +308,9 @@ public class ShuffleServerGrpcClient extends GrpcClient implements ShuffleServer
       List<ShuffleData> shuffleData = Lists.newArrayList();
       int size = 0;
       int blockNum = 0;
+      int shuffleId = stb.getKey();
+      List<Integer> partitionIds = new ArrayList<>();
+
       for (Map.Entry<Integer, List<ShuffleBlockInfo>> ptb : stb.getValue().entrySet()) {
         List<ShuffleBlock> shuffleBlocks = Lists.newArrayList();
         for (ShuffleBlockInfo sbi : ptb.getValue()) {
@@ -260,33 +327,42 @@ public class ShuffleServerGrpcClient extends GrpcClient implements ShuffleServer
         shuffleData.add(ShuffleData.newBuilder().setPartitionId(ptb.getKey())
             .addAllBlock(shuffleBlocks)
             .build());
+        partitionIds.add(ptb.getKey());
       }
 
       final int allocateSize = size;
       final int finalBlockNum = blockNum;
       try {
         RetryUtils.retry(() -> {
-          long requireId = requirePreAllocation(allocateSize, request.getRetryMax(), request.getRetryIntervalMax());
+          long requireId = requirePreAllocation(
+              appId,
+              shuffleId,
+              partitionIds,
+              allocateSize,
+              request.getRetryMax() / maxRetryAttempts,
+              request.getRetryIntervalMax()
+          );
           if (requireId == FAILED_REQUIRE_ID) {
             throw new RssException(String.format(
                 "requirePreAllocation failed! size[%s], host[%s], port[%s]", allocateSize, host, port));
           }
+          long start = System.currentTimeMillis();
           SendShuffleDataRequest rpcRequest = SendShuffleDataRequest.newBuilder()
               .setAppId(appId)
               .setShuffleId(stb.getKey())
               .setRequireBufferId(requireId)
               .addAllShuffleData(shuffleData)
+              .setTimestamp(start)
               .build();
-          long start = System.currentTimeMillis();
           SendShuffleDataResponse response = getBlockingStub().sendShuffleData(rpcRequest);
           LOG.info("Do sendShuffleData to {}:{} rpc cost:" + (System.currentTimeMillis() - start)
               + " ms for " + allocateSize + " bytes with " + finalBlockNum + " blocks", host, port);
-          if (response.getStatus() != StatusCode.SUCCESS) {
+          if (response.getStatus() != RssProtos.StatusCode.SUCCESS) {
             String msg = "Can't send shuffle data with " + finalBlockNum
                 + " blocks to " + host + ":" + port
                 + ", statusCode=" + response.getStatus()
                 + ", errorMsg:" + response.getRetMsg();
-            if (response.getStatus() == StatusCode.NO_REGISTER) {
+            if (response.getStatus() == RssProtos.StatusCode.NO_REGISTER) {
               throw new NotRetryException(msg);
             } else {
               throw new RssException(msg);
@@ -303,9 +379,9 @@ public class ShuffleServerGrpcClient extends GrpcClient implements ShuffleServer
 
     RssSendShuffleDataResponse response;
     if (isSuccessful) {
-      response = new RssSendShuffleDataResponse(ResponseStatusCode.SUCCESS);
+      response = new RssSendShuffleDataResponse(StatusCode.SUCCESS);
     } else {
-      response = new RssSendShuffleDataResponse(ResponseStatusCode.INTERNAL_ERROR);
+      response = new RssSendShuffleDataResponse(StatusCode.INTERNAL_ERROR);
     }
     return response;
   }
@@ -315,14 +391,14 @@ public class ShuffleServerGrpcClient extends GrpcClient implements ShuffleServer
     ShuffleCommitResponse rpcResponse = doSendCommit(request.getAppId(), request.getShuffleId());
 
     RssSendCommitResponse response;
-    if (rpcResponse.getStatus() != StatusCode.SUCCESS) {
+    if (rpcResponse.getStatus() != RssProtos.StatusCode.SUCCESS) {
       String msg = "Can't commit shuffle data to " + host + ":" + port
           + " for [appId=" + request.getAppId() + ", shuffleId=" + request.getShuffleId() + "], "
           + "errorMsg:" + rpcResponse.getRetMsg();
       LOG.error(msg);
       throw new RssException(msg);
     } else {
-      response = new RssSendCommitResponse(ResponseStatusCode.SUCCESS);
+      response = new RssSendCommitResponse(StatusCode.SUCCESS);
       response.setCommitCount(rpcResponse.getCommitCount());
     }
     return response;
@@ -331,14 +407,14 @@ public class ShuffleServerGrpcClient extends GrpcClient implements ShuffleServer
   @Override
   public RssAppHeartBeatResponse sendHeartBeat(RssAppHeartBeatRequest request) {
     AppHeartBeatResponse appHeartBeatResponse = doSendHeartBeat(request.getAppId(), request.getTimeoutMs());
-    if (appHeartBeatResponse.getStatus() != StatusCode.SUCCESS) {
+    if (appHeartBeatResponse.getStatus() != RssProtos.StatusCode.SUCCESS) {
       String msg = "Can't send heartbeat to " + host + ":" + port
           + " for [appId=" + request.getAppId() + ", timeout=" + request.getTimeoutMs() + "ms], "
           + "errorMsg:" + appHeartBeatResponse.getRetMsg();
       LOG.error(msg);
-      return new RssAppHeartBeatResponse(ResponseStatusCode.INTERNAL_ERROR);
+      return new RssAppHeartBeatResponse(StatusCode.INTERNAL_ERROR);
     } else {
-      return new RssAppHeartBeatResponse(ResponseStatusCode.SUCCESS);
+      return new RssAppHeartBeatResponse(StatusCode.SUCCESS);
     }
   }
 
@@ -350,7 +426,7 @@ public class ShuffleServerGrpcClient extends GrpcClient implements ShuffleServer
     FinishShuffleResponse rpcResponse = getBlockingStub().finishShuffle(rpcRequest);
 
     RssFinishShuffleResponse response;
-    if (rpcResponse.getStatus() != StatusCode.SUCCESS) {
+    if (rpcResponse.getStatus() != RssProtos.StatusCode.SUCCESS) {
       String msg = "Can't finish shuffle process to " + host + ":" + port
           + " for [appId=" + request.getAppId() + ", shuffleId=" + request.getShuffleId() + "], "
           + "errorMsg:" + rpcResponse.getRetMsg();
@@ -361,7 +437,7 @@ public class ShuffleServerGrpcClient extends GrpcClient implements ShuffleServer
           + request.getShuffleId() + "]";
       LOG.info("FinishShuffle to {}:{} for {} cost {} ms", host, port, requestInfo,
           System.currentTimeMillis() - start);
-      response = new RssFinishShuffleResponse(ResponseStatusCode.SUCCESS);
+      response = new RssFinishShuffleResponse(StatusCode.SUCCESS);
     }
     return response;
   }
@@ -388,11 +464,11 @@ public class ShuffleServerGrpcClient extends GrpcClient implements ShuffleServer
         .build();
     ReportShuffleResultResponse rpcResponse = doReportShuffleResult(recRequest);
 
-    StatusCode statusCode = rpcResponse.getStatus();
+    RssProtos.StatusCode statusCode = rpcResponse.getStatus();
     RssReportShuffleResultResponse response;
     switch (statusCode) {
       case SUCCESS:
-        response = new RssReportShuffleResultResponse(ResponseStatusCode.SUCCESS);
+        response = new RssReportShuffleResultResponse(StatusCode.SUCCESS);
         break;
       default:
         String msg = "Can't report shuffle result to " + host + ":" + port
@@ -429,13 +505,13 @@ public class ShuffleServerGrpcClient extends GrpcClient implements ShuffleServer
         .setPartitionId(request.getPartitionId())
         .build();
     GetShuffleResultResponse rpcResponse = getBlockingStub().getShuffleResult(rpcRequest);
-    StatusCode statusCode = rpcResponse.getStatus();
+    RssProtos.StatusCode statusCode = rpcResponse.getStatus();
 
     RssGetShuffleResultResponse response;
     switch (statusCode) {
       case SUCCESS:
         try {
-          response = new RssGetShuffleResultResponse(ResponseStatusCode.SUCCESS,
+          response = new RssGetShuffleResultResponse(StatusCode.SUCCESS,
               rpcResponse.getSerializedBitmap().toByteArray());
         } catch (Exception e) {
           throw new RuntimeException(e);
@@ -461,13 +537,13 @@ public class ShuffleServerGrpcClient extends GrpcClient implements ShuffleServer
         .addAllPartitions(request.getPartitions())
         .build();
     GetShuffleResultForMultiPartResponse rpcResponse = getBlockingStub().getShuffleResultForMultiPart(rpcRequest);
-    StatusCode statusCode = rpcResponse.getStatus();
+    RssProtos.StatusCode statusCode = rpcResponse.getStatus();
 
     RssGetShuffleResultResponse response;
     switch (statusCode) {
       case SUCCESS:
         try {
-          response = new RssGetShuffleResultResponse(ResponseStatusCode.SUCCESS,
+          response = new RssGetShuffleResultResponse(StatusCode.SUCCESS,
               rpcResponse.getSerializedBitmap().toByteArray());
         } catch (Exception e) {
           throw new RuntimeException(e);
@@ -486,6 +562,7 @@ public class ShuffleServerGrpcClient extends GrpcClient implements ShuffleServer
 
   @Override
   public RssGetShuffleDataResponse getShuffleData(RssGetShuffleDataRequest request) {
+    long start = System.currentTimeMillis();
     GetLocalShuffleDataRequest rpcRequest = GetLocalShuffleDataRequest
         .newBuilder()
         .setAppId(request.getAppId())
@@ -495,21 +572,21 @@ public class ShuffleServerGrpcClient extends GrpcClient implements ShuffleServer
         .setPartitionNum(request.getPartitionNum())
         .setOffset(request.getOffset())
         .setLength(request.getLength())
+        .setTimestamp(start)
         .build();
-    long start = System.currentTimeMillis();
     GetLocalShuffleDataResponse rpcResponse = getBlockingStub().getLocalShuffleData(rpcRequest);
     String requestInfo = "appId[" + request.getAppId() + "], shuffleId["
         + request.getShuffleId() + "], partitionId[" + request.getPartitionId() + "]";
     LOG.info("GetShuffleData from {}:{} for {} cost {} ms", host, port, requestInfo,
         System.currentTimeMillis() - start);
 
-    StatusCode statusCode = rpcResponse.getStatus();
+    RssProtos.StatusCode statusCode = rpcResponse.getStatus();
 
     RssGetShuffleDataResponse response;
     switch (statusCode) {
       case SUCCESS:
         response = new RssGetShuffleDataResponse(
-            ResponseStatusCode.SUCCESS, rpcResponse.getData().toByteArray());
+            StatusCode.SUCCESS, rpcResponse.getData().toByteArray());
 
         break;
       default:
@@ -538,13 +615,13 @@ public class ShuffleServerGrpcClient extends GrpcClient implements ShuffleServer
     LOG.info("GetShuffleIndex from {}:{} for {} cost {} ms", host, port,
         requestInfo, System.currentTimeMillis() - start);
 
-    StatusCode statusCode = rpcResponse.getStatus();
+    RssProtos.StatusCode statusCode = rpcResponse.getStatus();
 
     RssGetShuffleIndexResponse response;
     switch (statusCode) {
       case SUCCESS:
         response = new RssGetShuffleIndexResponse(
-            ResponseStatusCode.SUCCESS, rpcResponse.getIndexData().toByteArray(), rpcResponse.getDataFileLen());
+            StatusCode.SUCCESS, rpcResponse.getIndexData().toByteArray(), rpcResponse.getDataFileLen());
 
         break;
       default:
@@ -559,6 +636,17 @@ public class ShuffleServerGrpcClient extends GrpcClient implements ShuffleServer
   @Override
   public RssGetInMemoryShuffleDataResponse getInMemoryShuffleData(
       RssGetInMemoryShuffleDataRequest request) {
+    long start = System.currentTimeMillis();
+    ByteString serializedTaskIdsBytes = ByteString.EMPTY;
+    try {
+      if (request.getExpectedTaskIds() != null) {
+        serializedTaskIdsBytes =
+            UnsafeByteOperations.unsafeWrap(RssUtils.serializeBitMap(request.getExpectedTaskIds()));
+      }
+    } catch (Exception e) {
+      throw new RssException("Errors on serializing task ids bitmap.", e);
+    }
+
     GetMemoryShuffleDataRequest rpcRequest = GetMemoryShuffleDataRequest
         .newBuilder()
         .setAppId(request.getAppId())
@@ -566,22 +654,23 @@ public class ShuffleServerGrpcClient extends GrpcClient implements ShuffleServer
         .setPartitionId(request.getPartitionId())
         .setLastBlockId(request.getLastBlockId())
         .setReadBufferSize(request.getReadBufferSize())
+        .setSerializedExpectedTaskIdsBitmap(serializedTaskIdsBytes)
+        .setTimestamp(start)
         .build();
 
-    long start = System.currentTimeMillis();
     GetMemoryShuffleDataResponse rpcResponse = getBlockingStub().getMemoryShuffleData(rpcRequest);
     String requestInfo = "appId[" + request.getAppId() + "], shuffleId["
         + request.getShuffleId() + "], partitionId[" + request.getPartitionId() + "]";
     LOG.info("GetInMemoryShuffleData from {}:{} for " + requestInfo + " cost "
         + (System.currentTimeMillis() - start) + " ms", host, port);
 
-    StatusCode statusCode = rpcResponse.getStatus();
+    RssProtos.StatusCode statusCode = rpcResponse.getStatus();
 
     RssGetInMemoryShuffleDataResponse response;
     switch (statusCode) {
       case SUCCESS:
         response = new RssGetInMemoryShuffleDataResponse(
-            ResponseStatusCode.SUCCESS, rpcResponse.getData().toByteArray(),
+            StatusCode.SUCCESS, rpcResponse.getData().toByteArray(),
             toBufferSegments(rpcResponse.getShuffleDataBlockSegmentsList()));
         break;
       default:
