@@ -17,11 +17,13 @@
 
 package org.apache.uniffle.client.response;
 
+import org.apache.uniffle.common.rpc.StatusCode;
+
 public class RssGetShuffleDataResponse extends ClientResponse {
 
   private final byte[] shuffleData;
 
-  public RssGetShuffleDataResponse(ResponseStatusCode statusCode, byte[] data) {
+  public RssGetShuffleDataResponse(StatusCode statusCode, byte[] data) {
     super(statusCode);
     this.shuffleData = data;
   }
