@@ -21,12 +21,14 @@ public class ShuffleDataReadEvent {
 
   private String appId;
   private int shuffleId;
+  private int partitionId;
   private int startPartition;
 
-  public ShuffleDataReadEvent(String appId, int shuffleId, int startPartition) {
+  public ShuffleDataReadEvent(String appId, int shuffleId, int partitionId, int startPartitionOfRange) {
     this.appId = appId;
     this.shuffleId = shuffleId;
-    this.startPartition = startPartition;
+    this.partitionId = partitionId;
+    this.startPartition = startPartitionOfRange;
   }
 
   public String getAppId() {
@@ -37,6 +39,10 @@ public class ShuffleDataReadEvent {
     return shuffleId;
   }
 
+  public int getPartitionId() {
+    return partitionId;
+  }
+
   public int getStartPartition() {
     return startPartition;
   }
