@@ -19,6 +19,7 @@ package org.apache.uniffle.server;
 
 import java.util.List;
 import java.util.Map;
+import java.util.Optional;
 import java.util.stream.Collectors;
 
 import com.google.common.collect.Lists;
@@ -29,17 +30,23 @@ import com.google.protobuf.UnsafeByteOperations;
 import io.grpc.Context;
 import io.grpc.Status;
 import io.grpc.stub.StreamObserver;
+import org.apache.commons.lang3.StringUtils;
+import org.roaringbitmap.longlong.Roaring64NavigableMap;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import org.apache.uniffle.common.BufferSegment;
 import org.apache.uniffle.common.PartitionRange;
 import org.apache.uniffle.common.RemoteStorageInfo;
+import org.apache.uniffle.common.ShuffleDataDistributionType;
 import org.apache.uniffle.common.ShuffleDataResult;
 import org.apache.uniffle.common.ShuffleIndexResult;
 import org.apache.uniffle.common.ShufflePartitionedBlock;
 import org.apache.uniffle.common.ShufflePartitionedData;
 import org.apache.uniffle.common.config.RssBaseConf;
+import org.apache.uniffle.common.exception.FileNotFoundException;
+import org.apache.uniffle.common.rpc.StatusCode;
+import org.apache.uniffle.common.util.RssUtils;
 import org.apache.uniffle.proto.RssProtos;
 import org.apache.uniffle.proto.RssProtos.AppHeartBeatRequest;
 import org.apache.uniffle.proto.RssProtos.AppHeartBeatResponse;
@@ -72,7 +79,10 @@ import org.apache.uniffle.proto.RssProtos.ShufflePartitionRange;
 import org.apache.uniffle.proto.RssProtos.ShuffleRegisterRequest;
 import org.apache.uniffle.proto.RssProtos.ShuffleRegisterResponse;
 import org.apache.uniffle.proto.ShuffleServerGrpc.ShuffleServerImplBase;
+import org.apache.uniffle.server.buffer.PreAllocatedBufferInfo;
+import org.apache.uniffle.storage.common.Storage;
 import org.apache.uniffle.storage.common.StorageReadMetrics;
+import org.apache.uniffle.storage.util.ShuffleStorageUtils;
 
 public class ShuffleServerGrpcService extends ShuffleServerImplBase {
 
@@ -83,25 +93,27 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
     this.shuffleServer = shuffleServer;
   }
 
-  public static RssProtos.StatusCode valueOf(StatusCode code) {
-    switch (code) {
-      case SUCCESS:
-        return RssProtos.StatusCode.SUCCESS;
-      case DOUBLE_REGISTER:
-        return RssProtos.StatusCode.DOUBLE_REGISTER;
-      case NO_BUFFER:
-        return RssProtos.StatusCode.NO_BUFFER;
-      case INVALID_STORAGE:
-        return RssProtos.StatusCode.INVALID_STORAGE;
-      case NO_REGISTER:
-        return RssProtos.StatusCode.NO_REGISTER;
-      case NO_PARTITION:
-        return RssProtos.StatusCode.NO_PARTITION;
-      case TIMEOUT:
-        return RssProtos.StatusCode.TIMEOUT;
-      default:
-        return RssProtos.StatusCode.INTERNAL_ERROR;
+  @Override
+  public void unregisterShuffle(RssProtos.ShuffleUnregisterRequest request,
+      StreamObserver<RssProtos.ShuffleUnregisterResponse> responseStreamObserver) {
+    String appId = request.getAppId();
+    int shuffleId = request.getShuffleId();
+
+    StatusCode result = StatusCode.SUCCESS;
+    String responseMessage = "OK";
+    try {
+      shuffleServer.getShuffleTaskManager().removeShuffleDataAsync(appId, shuffleId);
+    } catch (Exception e) {
+      result = StatusCode.INTERNAL_ERROR;
     }
+
+    RssProtos.ShuffleUnregisterResponse reply = RssProtos.ShuffleUnregisterResponse
+        .newBuilder()
+        .setStatus(result.toProto())
+        .setRetMsg(responseMessage)
+        .build();
+    responseStreamObserver.onNext(reply);
+    responseStreamObserver.onCompleted();
   }
 
   @Override
@@ -114,6 +126,14 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
     String remoteStoragePath = req.getRemoteStorage().getPath();
     String user = req.getUser();
 
+    ShuffleDataDistributionType shuffleDataDistributionType =
+            ShuffleDataDistributionType.valueOf(
+                Optional
+                    .ofNullable(req.getShuffleDataDistribution())
+                    .orElse(RssProtos.DataDistribution.NORMAL)
+                    .name()
+            );
+
     Map<String, String> remoteStorageConf = req
         .getRemoteStorage()
         .getRemoteStorageConfList()
@@ -132,12 +152,13 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
             shuffleId,
             partitionRanges,
             new RemoteStorageInfo(remoteStoragePath, remoteStorageConf),
-            user
+            user,
+            shuffleDataDistributionType
         );
 
     reply = ShuffleRegisterResponse
         .newBuilder()
-        .setStatus(valueOf(result))
+        .setStatus(result.toProto())
         .build();
     responseObserver.onNext(reply);
     responseObserver.onCompleted();
@@ -151,6 +172,21 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
     String appId = req.getAppId();
     int shuffleId = req.getShuffleId();
     long requireBufferId = req.getRequireBufferId();
+    long timestamp = req.getTimestamp();
+    if (timestamp > 0) {
+      /*
+      * Here we record the transport time, but we don't consider the impact of data size on transport time.
+      * The amount of data will not cause great fluctuations in latency. For example, 100K costs 1ms,
+      * and 1M costs 10ms. This seems like a normal fluctuation, but it may rise to 10s when the server load is high.
+      * In addition, we need to pay attention to that the time of the client machine and the machine
+      * time of the Shuffle Server should be kept in sync. TransportTime is accurate only if this condition is met.
+      * */
+      long transportTime = System.currentTimeMillis() - timestamp;
+      if (transportTime > 0) {
+        shuffleServer.getGrpcMetrics().recordTransportTime(
+            ShuffleServerGrpcMetrics.SEND_SHUFFLE_DATA_METHOD, transportTime);
+      }
+    }
     int requireSize = shuffleServer
         .getShuffleTaskManager().getRequireBufferSize(requireBufferId);
 
@@ -158,7 +194,9 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
     String responseMessage = "OK";
     if (req.getShuffleDataCount() > 0) {
       ShuffleServerMetrics.counterTotalReceivedDataSize.inc(requireSize);
-      boolean isPreAllocated = shuffleServer.getShuffleTaskManager().isPreAllocated(requireBufferId);
+      ShuffleTaskManager manager = shuffleServer.getShuffleTaskManager();
+      PreAllocatedBufferInfo info = manager.getAndRemovePreAllocatedBuffer(requireBufferId);
+      boolean isPreAllocated = info != null;
       if (!isPreAllocated) {
         String errorMsg = "Can't find requireBufferId[" + requireBufferId + "] for appId[" + appId
             + "], shuffleId[" + shuffleId + "]";
@@ -166,7 +204,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
         responseMessage = errorMsg;
         reply = SendShuffleDataResponse
             .newBuilder()
-            .setStatus(valueOf(StatusCode.INTERNAL_ERROR))
+            .setStatus(StatusCode.INTERNAL_ERROR.toProto())
             .setRetMsg(responseMessage)
             .build();
         responseObserver.onNext(reply);
@@ -175,13 +213,12 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
       }
       final long start = System.currentTimeMillis();
       List<ShufflePartitionedData> shufflePartitionedData = toPartitionedData(req);
+      long alreadyReleasedSize = 0;
       for (ShufflePartitionedData spd : shufflePartitionedData) {
         String shuffleDataInfo = "appId[" + appId + "], shuffleId[" + shuffleId
             + "], partitionId[" + spd.getPartitionId() + "]";
         try {
-          ret = shuffleServer
-              .getShuffleTaskManager()
-              .cacheShuffleData(appId, shuffleId, isPreAllocated, spd);
+          ret = manager.cacheShuffleData(appId, shuffleId, isPreAllocated, spd);
           if (ret != StatusCode.SUCCESS) {
             String errorMsg = "Error happened when shuffleEngine.write for "
                 + shuffleDataInfo + ", statusCode=" + ret;
@@ -189,11 +226,11 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
             responseMessage = errorMsg;
             break;
           } else {
-            // remove require bufferId, the memory should be updated already
-            shuffleServer
-                .getShuffleTaskManager().removeRequireBufferId(requireBufferId);
-            shuffleServer.getShuffleTaskManager().updateCachedBlockIds(
-                appId, shuffleId, spd.getBlockList());
+            long toReleasedSize = spd.getTotalBlockSize();
+            // after each cacheShuffleData call, the `preAllocatedSize` is updated timely.
+            manager.releasePreAllocatedSize(toReleasedSize);
+            alreadyReleasedSize += toReleasedSize;
+            manager.updateCachedBlockIds(appId, shuffleId, spd.getPartitionId(), spd.getBlockList());
           }
         } catch (Exception e) {
           String errorMsg = "Error happened when shuffleEngine.write for "
@@ -204,14 +241,22 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
           break;
         }
       }
-      reply = SendShuffleDataResponse.newBuilder().setStatus(valueOf(ret)).setRetMsg(responseMessage).build();
+      // since the required buffer id is only used once, the shuffle client would try to require another buffer whether
+      // current connection succeeded or not. Therefore, the preAllocatedBuffer is first get and removed, then after
+      // cacheShuffleData finishes, the preAllocatedSize should be updated accordingly.
+      if (info.getRequireSize() > alreadyReleasedSize) {
+        manager.releasePreAllocatedSize(info.getRequireSize() - alreadyReleasedSize);
+      }
+      reply = SendShuffleDataResponse.newBuilder().setStatus(ret.toProto()).setRetMsg(responseMessage).build();
+      long costTime = System.currentTimeMillis() - start;
+      shuffleServer.getGrpcMetrics().recordProcessTime(ShuffleServerGrpcMetrics.SEND_SHUFFLE_DATA_METHOD, costTime);
       LOG.debug("Cache Shuffle Data for appId[" + appId + "], shuffleId[" + shuffleId
-          + "], cost " + (System.currentTimeMillis() - start)
+          + "], cost " + costTime
           + " ms with " + shufflePartitionedData.size() + " blocks and " + requireSize + " bytes");
     } else {
       reply = SendShuffleDataResponse
           .newBuilder()
-          .setStatus(valueOf(StatusCode.INTERNAL_ERROR))
+          .setStatus(StatusCode.INTERNAL_ERROR.toProto())
           .setRetMsg("No data in request")
           .build();
     }
@@ -248,7 +293,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
     reply = ShuffleCommitResponse
         .newBuilder()
         .setCommitCount(commitCount)
-        .setStatus(valueOf(status))
+        .setStatus(status.toProto())
         .setRetMsg(msg)
         .build();
     responseObserver.onNext(reply);
@@ -280,7 +325,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
     FinishShuffleResponse response =
         FinishShuffleResponse
             .newBuilder()
-            .setStatus(valueOf(status))
+            .setStatus(status.toProto())
             .setRetMsg(msg).build();
     responseObserver.onNext(response);
     responseObserver.onCompleted();
@@ -289,7 +334,22 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
   @Override
   public void requireBuffer(RequireBufferRequest request,
       StreamObserver<RequireBufferResponse> responseObserver) {
-    long requireBufferId = shuffleServer.getShuffleTaskManager().requireBuffer(request.getRequireSize());
+    String appId = request.getAppId();
+    long requireBufferId;
+    if (StringUtils.isEmpty(appId)) {
+      // To be compatible with older client version
+      requireBufferId = shuffleServer.getShuffleTaskManager().requireBuffer(
+          request.getRequireSize()
+      );
+    } else {
+      requireBufferId = shuffleServer.getShuffleTaskManager().requireBuffer(
+          appId,
+          request.getShuffleId(),
+          request.getPartitionIdsList(),
+          request.getRequireSize()
+      );
+    }
+
     StatusCode status = StatusCode.SUCCESS;
     if (requireBufferId == -1) {
       status = StatusCode.NO_BUFFER;
@@ -298,7 +358,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
     RequireBufferResponse response =
         RequireBufferResponse
             .newBuilder()
-            .setStatus(valueOf(status))
+            .setStatus(status.toProto())
             .setRequireBufferId(requireBufferId)
             .build();
     responseObserver.onNext(response);
@@ -313,7 +373,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
     AppHeartBeatResponse response = AppHeartBeatResponse
         .newBuilder()
         .setRetMsg("")
-        .setStatus(valueOf(StatusCode.SUCCESS))
+        .setStatus(StatusCode.SUCCESS.toProto())
         .build();
 
     if (Context.current().isCancelled()) {
@@ -349,7 +409,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
       LOG.error("Error happened when report shuffle result for " + requestInfo, e);
     }
 
-    reply = ReportShuffleResultResponse.newBuilder().setStatus(valueOf(status)).setRetMsg(msg).build();
+    reply = ReportShuffleResultResponse.newBuilder().setStatus(status.toProto()).setRetMsg(msg).build();
     responseObserver.onNext(reply);
     responseObserver.onCompleted();
   }
@@ -384,7 +444,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
     }
 
     reply = GetShuffleResultResponse.newBuilder()
-        .setStatus(valueOf(status))
+        .setStatus(status.toProto())
         .setRetMsg(msg)
         .setSerializedBitmap(serializedBlockIdsBytes)
         .build();
@@ -423,7 +483,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
     }
 
     reply = GetShuffleResultForMultiPartResponse.newBuilder()
-        .setStatus(valueOf(status))
+        .setStatus(status.toProto())
         .setRetMsg(msg)
         .setSerializedBitmap(serializedBlockIdsBytes)
         .build();
@@ -441,6 +501,14 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
     int partitionNum = request.getPartitionNum();
     long offset = request.getOffset();
     int length = request.getLength();
+    long timestamp = request.getTimestamp();
+    if (timestamp > 0) {
+      long transportTime = System.currentTimeMillis() - timestamp;
+      if (transportTime > 0) {
+        shuffleServer.getGrpcMetrics().recordTransportTime(
+            ShuffleServerGrpcMetrics.GET_SHUFFLE_DATA_METHOD, transportTime);
+      }
+    }
     String storageType = shuffleServer.getShuffleServerConf().get(RssBaseConf.RSS_STORAGE_TYPE);
     StatusCode status = StatusCode.SUCCESS;
     String msg = "OK";
@@ -449,9 +517,15 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
     String requestInfo = "appId[" + appId + "], shuffleId[" + shuffleId + "], partitionId["
         + partitionId + "]" + "offset[" + offset + "]" + "length[" + length + "]";
 
-    shuffleServer.getStorageManager()
-        .selectStorage(new ShuffleDataReadEvent(appId, shuffleId, partitionId))
-        .updateReadMetrics(new StorageReadMetrics(appId, shuffleId));
+    int[] range = ShuffleStorageUtils.getPartitionRange(partitionId, partitionNumPerRange, partitionNum);
+    Storage storage = shuffleServer
+        .getStorageManager()
+        .selectStorage(
+            new ShuffleDataReadEvent(appId, shuffleId, partitionId, range[0])
+        );
+    if (storage != null) {
+      storage.updateReadMetrics(new StorageReadMetrics(appId, shuffleId));
+    }
 
     if (shuffleServer.getShuffleBufferManager().requireReadMemoryWithRetry(length)) {
       try {
@@ -462,10 +536,12 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
         ShuffleServerMetrics.counterTotalReadTime.inc(readTime);
         ShuffleServerMetrics.counterTotalReadDataSize.inc(sdr.getData().length);
         ShuffleServerMetrics.counterTotalReadLocalDataFileSize.inc(sdr.getData().length);
+        shuffleServer.getGrpcMetrics().recordProcessTime(
+            ShuffleServerGrpcMetrics.GET_SHUFFLE_DATA_METHOD, readTime);
         LOG.info("Successfully getShuffleData cost {} ms for shuffle"
             + " data with {}", readTime, requestInfo);
         reply = GetLocalShuffleDataResponse.newBuilder()
-            .setStatus(valueOf(status))
+            .setStatus(status.toProto())
             .setRetMsg(msg)
             .setData(UnsafeByteOperations.unsafeWrap(sdr.getData()))
             .build();
@@ -474,7 +550,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
         msg = "Error happened when get shuffle data for " + requestInfo + ", " + e.getMessage();
         LOG.error(msg, e);
         reply = GetLocalShuffleDataResponse.newBuilder()
-            .setStatus(valueOf(status))
+            .setStatus(status.toProto())
             .setRetMsg(msg)
             .build();
       } finally {
@@ -485,7 +561,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
       msg = "Can't require memory to get shuffle data";
       LOG.error(msg + " for " + requestInfo);
       reply = GetLocalShuffleDataResponse.newBuilder()
-          .setStatus(valueOf(status))
+          .setStatus(status.toProto())
           .setRetMsg(msg)
           .build();
     }
@@ -507,10 +583,12 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
     String requestInfo = "appId[" + appId + "], shuffleId[" + shuffleId + "], partitionId["
         + partitionId + "]";
 
-    shuffleServer.getStorageManager()
-        .selectStorage(new ShuffleDataReadEvent(appId, shuffleId, partitionId))
-        .updateReadMetrics(new StorageReadMetrics(appId, shuffleId));
-
+    int[] range = ShuffleStorageUtils.getPartitionRange(partitionId, partitionNumPerRange, partitionNum);
+    Storage storage = shuffleServer.getStorageManager()
+        .selectStorage(new ShuffleDataReadEvent(appId, shuffleId, partitionId, range[0]));
+    if (storage != null) {
+      storage.updateReadMetrics(new StorageReadMetrics(appId, shuffleId));
+    }
     // Index file is expected small size and won't cause oom problem with the assumed size. An index segment is 40B,
     // with the default size - 2MB, it can support 50k blocks for shuffle data.
     long assumedFileSize = shuffleServer
@@ -526,7 +604,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
         ShuffleServerMetrics.counterTotalReadDataSize.inc(data.length);
         ShuffleServerMetrics.counterTotalReadLocalIndexFileSize.inc(data.length);
         GetLocalShuffleIndexResponse.Builder builder = GetLocalShuffleIndexResponse.newBuilder()
-            .setStatus(valueOf(status))
+            .setStatus(status.toProto())
             .setRetMsg(msg);
         LOG.info("Successfully getShuffleIndex cost {} ms for {}"
             + " bytes with {}", readTime, data.length, requestInfo);
@@ -534,12 +612,18 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
         builder.setIndexData(UnsafeByteOperations.unsafeWrap(data));
         builder.setDataFileLen(shuffleIndexResult.getDataFileLen());
         reply = builder.build();
+      } catch (FileNotFoundException indexFileNotFoundException) {
+        LOG.warn("Index file for {} is not found, maybe the data has been flushed to cold storage.",
+            requestInfo, indexFileNotFoundException);
+        reply = GetLocalShuffleIndexResponse.newBuilder()
+            .setStatus(status.toProto())
+            .build();
       } catch (Exception e) {
         status = StatusCode.INTERNAL_ERROR;
         msg = "Error happened when get shuffle index for " + requestInfo + ", " + e.getMessage();
         LOG.error(msg, e);
         reply = GetLocalShuffleIndexResponse.newBuilder()
-            .setStatus(valueOf(status))
+            .setStatus(status.toProto())
             .setRetMsg(msg)
             .build();
       } finally {
@@ -550,7 +634,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
       msg = "Can't require memory to get shuffle index";
       LOG.error(msg + " for " + requestInfo);
       reply = GetLocalShuffleIndexResponse.newBuilder()
-          .setStatus(valueOf(status))
+          .setStatus(status.toProto())
           .setRetMsg(msg)
           .build();
     }
@@ -566,6 +650,15 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
     int partitionId = request.getPartitionId();
     long blockId = request.getLastBlockId();
     int readBufferSize = request.getReadBufferSize();
+    long timestamp = request.getTimestamp();
+
+    if (timestamp > 0) {
+      long transportTime = System.currentTimeMillis() - timestamp;
+      if (transportTime > 0) {
+        shuffleServer.getGrpcMetrics().recordTransportTime(
+            ShuffleServerGrpcMetrics.GET_MEMORY_SHUFFLE_DATA_METHOD, transportTime);
+      }
+    }
     long start = System.currentTimeMillis();
     StatusCode status = StatusCode.SUCCESS;
     String msg = "OK";
@@ -576,8 +669,23 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
     // todo: if can get the exact memory size?
     if (shuffleServer.getShuffleBufferManager().requireReadMemoryWithRetry(readBufferSize)) {
       try {
-        ShuffleDataResult shuffleDataResult = shuffleServer.getShuffleTaskManager()
-            .getInMemoryShuffleData(appId, shuffleId, partitionId, blockId, readBufferSize);
+        Roaring64NavigableMap expectedTaskIds = null;
+        if (request.getSerializedExpectedTaskIdsBitmap() != null
+            && !request.getSerializedExpectedTaskIdsBitmap().isEmpty()) {
+          expectedTaskIds = RssUtils.deserializeBitMap(
+              request.getSerializedExpectedTaskIdsBitmap().toByteArray()
+          );
+        }
+        ShuffleDataResult shuffleDataResult = shuffleServer
+            .getShuffleTaskManager()
+            .getInMemoryShuffleData(
+                appId,
+                shuffleId,
+                partitionId,
+                blockId,
+                readBufferSize,
+                expectedTaskIds
+            );
         byte[] data = new byte[]{};
         List<BufferSegment> bufferSegments = Lists.newArrayList();
         if (shuffleDataResult != null) {
@@ -586,11 +694,14 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
           ShuffleServerMetrics.counterTotalReadDataSize.inc(data.length);
           ShuffleServerMetrics.counterTotalReadMemoryDataSize.inc(data.length);
         }
+        long costTime = System.currentTimeMillis() - start;
+        shuffleServer.getGrpcMetrics().recordProcessTime(
+            ShuffleServerGrpcMetrics.GET_MEMORY_SHUFFLE_DATA_METHOD, costTime);
         LOG.info("Successfully getInMemoryShuffleData cost {} ms with {} bytes shuffle"
-            + " data for {}", (System.currentTimeMillis() - start), data.length, requestInfo);
+            + " data for {}", costTime, data.length, requestInfo);
 
         reply = GetMemoryShuffleDataResponse.newBuilder()
-            .setStatus(valueOf(status))
+            .setStatus(status.toProto())
             .setRetMsg(msg)
             .setData(UnsafeByteOperations.unsafeWrap(data))
             .addAllShuffleDataBlockSegments(toShuffleDataBlockSegments(bufferSegments))
@@ -603,7 +714,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
         reply = GetMemoryShuffleDataResponse.newBuilder()
             .setData(UnsafeByteOperations.unsafeWrap(new byte[]{}))
             .addAllShuffleDataBlockSegments(Lists.newArrayList())
-            .setStatus(valueOf(status))
+            .setStatus(status.toProto())
             .setRetMsg(msg)
             .build();
       } finally {
@@ -616,7 +727,7 @@ public class ShuffleServerGrpcService extends ShuffleServerImplBase {
       reply = GetMemoryShuffleDataResponse.newBuilder()
           .setData(UnsafeByteOperations.unsafeWrap(new byte[]{}))
           .addAllShuffleDataBlockSegments(Lists.newArrayList())
-          .setStatus(valueOf(status))
+          .setStatus(status.toProto())
           .setRetMsg(msg)
           .build();
     }
