@@ -32,18 +32,20 @@ import com.google.common.collect.Range;
 import com.google.common.collect.RangeMap;
 import com.google.common.collect.Sets;
 import com.google.common.collect.TreeRangeMap;
+import org.roaringbitmap.longlong.Roaring64NavigableMap;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import org.apache.uniffle.common.ShuffleDataResult;
 import org.apache.uniffle.common.ShufflePartitionedData;
+import org.apache.uniffle.common.rpc.StatusCode;
 import org.apache.uniffle.common.util.Constants;
 import org.apache.uniffle.common.util.RssUtils;
+import org.apache.uniffle.common.util.TripleFunction;
 import org.apache.uniffle.server.ShuffleDataFlushEvent;
 import org.apache.uniffle.server.ShuffleFlushManager;
 import org.apache.uniffle.server.ShuffleServerConf;
 import org.apache.uniffle.server.ShuffleServerMetrics;
-import org.apache.uniffle.server.StatusCode;
 
 public class ShuffleBufferManager {
 
@@ -57,6 +59,12 @@ public class ShuffleBufferManager {
   private long lowWaterMark;
   private boolean bufferFlushEnabled;
   private long bufferFlushThreshold;
+  // when shuffle buffer manager flushes data, shuffles with data size < shuffleFlushThreshold is kept in memory to
+  // reduce small I/Os to persistent storage, especially for local HDDs.
+  private long shuffleFlushThreshold;
+  // Huge partition vars
+  private long hugePartitionSizeThreshold;
+  private long hugePartitionMemoryLimitSize;
 
   protected long bufferSize = 0;
   protected AtomicLong preAllocatedSize = new AtomicLong(0L);
@@ -80,6 +88,11 @@ public class ShuffleBufferManager {
         * conf.get(ShuffleServerConf.SERVER_MEMORY_SHUFFLE_LOWWATERMARK_PERCENTAGE));
     this.bufferFlushEnabled = conf.getBoolean(ShuffleServerConf.SINGLE_BUFFER_FLUSH_ENABLED);
     this.bufferFlushThreshold = conf.getLong(ShuffleServerConf.SINGLE_BUFFER_FLUSH_THRESHOLD);
+    this.shuffleFlushThreshold = conf.getLong(ShuffleServerConf.SERVER_SHUFFLE_FLUSH_THRESHOLD);
+    this.hugePartitionSizeThreshold = conf.getSizeAsBytes(ShuffleServerConf.HUGE_PARTITION_SIZE_THRESHOLD);
+    this.hugePartitionMemoryLimitSize = Math.round(
+        capacity * conf.get(ShuffleServerConf.HUGE_PARTITION_MEMORY_USAGE_LIMITATION_RATIO)
+    );
   }
 
   public StatusCode registerBuffer(String appId, int shuffleId, int startPartition, int endPartition) {
@@ -88,6 +101,7 @@ public class ShuffleBufferManager {
     shuffleIdToBuffers.putIfAbsent(shuffleId, TreeRangeMap.create());
     RangeMap<Integer, ShuffleBuffer> bufferRangeMap = shuffleIdToBuffers.get(shuffleId);
     if (bufferRangeMap.get(startPartition) == null) {
+      ShuffleServerMetrics.counterTotalPartitionNum.inc();
       ShuffleServerMetrics.gaugeTotalPartitionNum.inc();
       bufferRangeMap.put(Range.closed(startPartition, endPartition), new ShuffleBuffer(bufferSize));
     } else {
@@ -98,10 +112,29 @@ public class ShuffleBufferManager {
     return StatusCode.SUCCESS;
   }
 
-  public StatusCode cacheShuffleData(String appId, int shuffleId,
-      boolean isPreAllocated, ShufflePartitionedData spd) {
+  // Only for tests
+  public StatusCode cacheShuffleData(
+      String appId,
+      int shuffleId,
+      boolean isPreAllocated,
+      ShufflePartitionedData spd) {
+    return cacheShuffleData(
+        appId,
+        shuffleId,
+        isPreAllocated,
+        spd,
+        null
+    );
+  }
+
+  public StatusCode cacheShuffleData(
+      String appId,
+      int shuffleId,
+      boolean isPreAllocated,
+      ShufflePartitionedData spd,
+      TripleFunction<String, Integer, Integer, Long> getPartitionDataSizeFunc) {
     if (!isPreAllocated && isFull()) {
-      LOG.warn("Got unexpect data, can't cache it because the space is full");
+      LOG.warn("Got unexpected data, can't cache it because the space is full");
       return StatusCode.NO_BUFFER;
     }
 
@@ -112,12 +145,21 @@ public class ShuffleBufferManager {
     }
 
     ShuffleBuffer buffer = entry.getValue();
-    int size = buffer.append(spd);
-    updateSize(size, isPreAllocated);
+    long size = buffer.append(spd);
+    if (!isPreAllocated) {
+      updateUsedMemory(size);
+    }
     updateShuffleSize(appId, shuffleId, size);
     synchronized (this) {
-      flushSingleBufferIfNecessary(buffer, appId, shuffleId,
-          entry.getKey().lowerEndpoint(), entry.getKey().upperEndpoint());
+      flushSingleBufferIfNecessary(
+          buffer,
+          appId,
+          shuffleId,
+          spd.getPartitionId(),
+          entry.getKey().lowerEndpoint(),
+          entry.getKey().upperEndpoint(),
+          getPartitionDataSizeFunc
+      );
       flushIfNecessary();
     }
     return StatusCode.SUCCESS;
@@ -130,7 +172,7 @@ public class ShuffleBufferManager {
     shuffleIdToSize.get(shuffleId).addAndGet(size);
   }
 
-  protected Entry<Range<Integer>, ShuffleBuffer> getShuffleBufferEntry(
+  public Entry<Range<Integer>, ShuffleBuffer> getShuffleBufferEntry(
       String appId, int shuffleId, int partitionId) {
     Map<Integer, RangeMap<Integer, ShuffleBuffer>> shuffleIdToBuffers = bufferPool.get(appId);
     if (shuffleIdToBuffers == null) {
@@ -150,6 +192,19 @@ public class ShuffleBufferManager {
   public ShuffleDataResult getShuffleData(
       String appId, int shuffleId, int partitionId, long blockId,
       int readBufferSize) {
+    return getShuffleData(
+        appId,
+        shuffleId,
+        partitionId,
+        blockId,
+        readBufferSize,
+        null
+    );
+  }
+
+  public ShuffleDataResult getShuffleData(
+      String appId, int shuffleId, int partitionId, long blockId,
+      int readBufferSize, Roaring64NavigableMap expectedTaskIds) {
     Map.Entry<Range<Integer>, ShuffleBuffer> entry = getShuffleBufferEntry(
         appId, shuffleId, partitionId);
     if (entry == null) {
@@ -160,21 +215,36 @@ public class ShuffleBufferManager {
     if (buffer == null) {
       return null;
     }
-    return buffer.getShuffleData(blockId, readBufferSize);
+    return buffer.getShuffleData(blockId, readBufferSize, expectedTaskIds);
   }
 
-  void flushSingleBufferIfNecessary(ShuffleBuffer buffer, String appId,
-      int shuffleId, int startPartition, int endPartition) {
-    // When we use multistorage and trigger single buffer flush, the buffer size should be bigger
+  void flushSingleBufferIfNecessary(
+      ShuffleBuffer buffer,
+      String appId,
+      int shuffleId,
+      int partitionId,
+      int startPartition,
+      int endPartition,
+      TripleFunction<String, Integer, Integer, Long> getPartitionDataSizeFunc) {
+    // When we use multi storage and trigger single buffer flush, the buffer size should be bigger
     // than rss.server.flush.cold.storage.threshold.size, otherwise cold storage will be useless.
     if (this.bufferFlushEnabled && buffer.getSize() > this.bufferFlushThreshold) {
       flushBuffer(buffer, appId, shuffleId, startPartition, endPartition);
+      return;
+    }
+
+    if (getPartitionDataSizeFunc != null
+        && getPartitionDataSizeFunc.accept(appId, shuffleId, partitionId) > hugePartitionSizeThreshold
+        && buffer.getSize() > this.bufferFlushThreshold) {
+      flushBuffer(buffer, appId, shuffleId, startPartition, endPartition);
+      return;
     }
   }
 
-  void flushIfNecessary() {
+  public void flushIfNecessary() {
     // if data size in buffer > highWaterMark, do the flush
     if (usedMemory.get() - preAllocatedSize.get() - inFlushSize.get() > highWaterMark) {
+      // todo: add a metric here to track how many times flush occurs.
       LOG.info("Start to flush with usedMemory[{}], preAllocatedSize[{}], inFlushSize[{}]",
           usedMemory.get(), preAllocatedSize.get(), inFlushSize.get());
       Map<String, Set<Integer>> pickedShuffle = pickFlushedShuffle();
@@ -194,9 +264,16 @@ public class ShuffleBufferManager {
   protected void flushBuffer(ShuffleBuffer buffer, String appId,
       int shuffleId, int startPartition, int endPartition) {
     ShuffleDataFlushEvent event =
-        buffer.toFlushEvent(appId, shuffleId, startPartition, endPartition,
-            () -> bufferPool.containsKey(appId));
+        buffer.toFlushEvent(
+            appId,
+            shuffleId,
+            startPartition,
+            endPartition,
+            () -> bufferPool.containsKey(appId),
+            shuffleFlushManager.getDataDistributionType(appId)
+        );
     if (event != null) {
+      event.addCleanupCallback(() -> releaseMemory(event.getSize(), true, false));
       updateShuffleSize(appId, shuffleId, -event.getSize());
       inFlushSize.addAndGet(event.getSize());
       ShuffleServerMetrics.gaugeInFlushBufferSize.set(inFlushSize.get());
@@ -209,21 +286,7 @@ public class ShuffleBufferManager {
     if (shuffleIdToBuffers == null) {
       return;
     }
-    // calculate released size
-    long size = 0;
-    for (RangeMap<Integer, ShuffleBuffer> rangeMap : shuffleIdToBuffers.values()) {
-      if (rangeMap != null) {
-        Collection<ShuffleBuffer> buffers = rangeMap.asMapOfRanges().values();
-        if (buffers != null) {
-          for (ShuffleBuffer buffer : buffers) {
-            ShuffleServerMetrics.gaugeTotalPartitionNum.dec();
-            size += buffer.getSize();
-          }
-        }
-      }
-    }
-    // release memory
-    releaseMemory(size, false, false);
+    removeBufferByShuffleId(appId, shuffleIdToBuffers.keySet());
     shuffleSizeMap.remove(appId);
     bufferPool.remove(appId);
   }
@@ -275,31 +338,37 @@ public class ShuffleBufferManager {
   }
 
   public boolean requireReadMemoryWithRetry(long size) {
+    ShuffleServerMetrics.counterTotalRequireReadMemoryNum.inc();
     for (int i = 0; i < retryNum; i++) {
       synchronized (this) {
         if (readDataMemory.get() + size < readCapacity) {
           readDataMemory.addAndGet(size);
+          ShuffleServerMetrics.gaugeReadBufferUsedSize.inc(size);
           return true;
         }
       }
       LOG.info("Can't require[" + size + "] for read data, current[" + readDataMemory.get()
           + "], capacity[" + readCapacity + "], re-try " + i + " times");
+      ShuffleServerMetrics.counterTotalRequireReadMemoryRetryNum.inc();
       try {
         Thread.sleep(1000);
       } catch (Exception e) {
         LOG.warn("Error happened when require memory", e);
       }
     }
+    ShuffleServerMetrics.counterTotalRequireReadMemoryFailedNum.inc();
     return false;
   }
 
   public void releaseReadMemory(long size) {
     if (readDataMemory.get() >= size) {
       readDataMemory.addAndGet(-size);
+      ShuffleServerMetrics.gaugeReadBufferUsedSize.dec(size);
     } else {
       LOG.warn("Current read memory[" + readDataMemory.get()
           + "] is less than released[" + size + "], set read memory to 0");
       readDataMemory.set(0L);
+      ShuffleServerMetrics.gaugeReadBufferUsedSize.set(0);
     }
   }
 
@@ -326,14 +395,10 @@ public class ShuffleBufferManager {
     }
   }
 
-  void updateSize(long delta, boolean isPreAllocated) {
-    if (isPreAllocated) {
-      releasePreAllocatedSize(delta);
-    } else {
-      // add size if not allocated
-      usedMemory.addAndGet(delta);
-      ShuffleServerMetrics.gaugeUsedBufferSize.set(usedMemory.get());
-    }
+  public void updateUsedMemory(long delta) {
+    // add size if not allocated
+    usedMemory.addAndGet(delta);
+    ShuffleServerMetrics.gaugeUsedBufferSize.set(usedMemory.get());
   }
 
   void requirePreAllocatedSize(long delta) {
@@ -341,7 +406,7 @@ public class ShuffleBufferManager {
     ShuffleServerMetrics.gaugeAllocatedBufferSize.set(preAllocatedSize.get());
   }
 
-  void releasePreAllocatedSize(long delta) {
+  public void releasePreAllocatedSize(long delta) {
     preAllocatedSize.addAndGet(-delta);
     ShuffleServerMetrics.gaugeAllocatedBufferSize.set(preAllocatedSize.get());
   }
@@ -412,25 +477,38 @@ public class ShuffleBufferManager {
 
     Map<String, Set<Integer>> pickedShuffle = Maps.newHashMap();
     // The algorithm here is to flush data size > highWaterMark - lowWaterMark
-    // the remain data in buffer maybe more than lowWaterMark
+    // the remaining data in buffer maybe more than lowWaterMark
     // because shuffle server is still receiving data, but it should be ok
     long expectedFlushSize = highWaterMark - lowWaterMark;
+    long atLeastFlushSizeIgnoreThreshold = expectedFlushSize >>> 1;
     long pickedFlushSize = 0L;
     int printIndex = 0;
+    int printIgnoreIndex = 0;
     int printMax = 10;
     for (Map.Entry<String, AtomicLong> entry : sizeList) {
       long size = entry.getValue().get();
-      pickedFlushSize += size;
       String appIdShuffleIdKey = entry.getKey();
-      addPickedShuffle(appIdShuffleIdKey, pickedShuffle);
-      // print detail picked info
-      if (printIndex < printMax) {
-        LOG.info("Pick application_shuffleId[{}] with {} bytes", appIdShuffleIdKey, size);
-        printIndex++;
-      }
-      if (pickedFlushSize > expectedFlushSize) {
-        LOG.info("Finish flush pick with {} bytes", pickedFlushSize);
-        break;
+      if (size > this.shuffleFlushThreshold || pickedFlushSize <= atLeastFlushSizeIgnoreThreshold) {
+        pickedFlushSize += size;
+        addPickedShuffle(appIdShuffleIdKey, pickedShuffle);
+        // print detail picked info
+        if (printIndex < printMax) {
+          LOG.info("Pick application_shuffleId[{}] with {} bytes", appIdShuffleIdKey, size);
+          printIndex++;
+        }
+        if (pickedFlushSize > expectedFlushSize) {
+          LOG.info("Finish flush pick with {} bytes", pickedFlushSize);
+          break;
+        }
+      } else {
+        // since shuffle size is ordered by size desc, we can skip process more shuffle data once some shuffle's size
+        // is less than threshold
+        if (printIgnoreIndex < printMax) {
+          LOG.info("Ignore application_shuffleId[{}] with {} bytes", appIdShuffleIdKey, size);
+          printIgnoreIndex++;
+        } else {
+          break;
+        }
       }
     }
     return pickedShuffle;
@@ -456,4 +534,48 @@ public class ShuffleBufferManager {
     Set<Integer> shuffleIdSet = pickedShuffle.get(appId);
     shuffleIdSet.add(shuffleId);
   }
+
+  public void removeBufferByShuffleId(String appId, Collection<Integer> shuffleIds) {
+    Map<Integer, RangeMap<Integer, ShuffleBuffer>> shuffleIdToBuffers = bufferPool.get(appId);
+    if (shuffleIdToBuffers == null) {
+      return;
+    }
+
+    Map<Integer, AtomicLong> shuffleIdToSizeMap = shuffleSizeMap.get(appId);
+    for (int shuffleId : shuffleIds) {
+      long size = 0;
+
+      RangeMap<Integer, ShuffleBuffer> bufferRangeMap = shuffleIdToBuffers.remove(shuffleId);
+      if (bufferRangeMap == null) {
+        continue;
+      }
+      Collection<ShuffleBuffer> buffers = bufferRangeMap.asMapOfRanges().values();
+      if (buffers != null) {
+        for (ShuffleBuffer buffer : buffers) {
+          ShuffleServerMetrics.gaugeTotalPartitionNum.dec();
+          size += buffer.getSize();
+        }
+      }
+      releaseMemory(size, false, false);
+      if (shuffleIdToSizeMap != null) {
+        shuffleIdToSizeMap.remove(shuffleId);
+      }
+    }
+  }
+
+  public boolean isHugePartition(long usedPartitionDataSize) {
+    return usedPartitionDataSize > hugePartitionSizeThreshold;
+  }
+
+  public boolean limitHugePartition(String appId, int shuffleId, int partitionId, long usedPartitionDataSize) {
+    if (usedPartitionDataSize > hugePartitionSizeThreshold) {
+      long memoryUsed = getShuffleBufferEntry(appId, shuffleId, partitionId).getValue().getSize();
+      if (memoryUsed > hugePartitionMemoryLimitSize) {
+        LOG.warn("AppId: {}, shuffleId: {}, partitionId: {}, memory used: {}, "
+            + "huge partition triggered memory limitation.", appId, shuffleId, partitionId, memoryUsed);
+        return true;
+      }
+    }
+    return false;
+  }
 }
