@@ -26,6 +26,7 @@ import org.roaringbitmap.longlong.Roaring64NavigableMap;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import org.apache.uniffle.common.ShuffleDataDistributionType;
 import org.apache.uniffle.common.ShuffleDataResult;
 import org.apache.uniffle.common.ShuffleDataSegment;
 import org.apache.uniffle.common.ShuffleIndexResult;
@@ -51,19 +52,36 @@ public class HdfsShuffleReadHandler extends DataSkippableReadHandler {
       int readBufferSize,
       Roaring64NavigableMap expectBlockIds,
       Roaring64NavigableMap processBlockIds,
-      Configuration conf) throws Exception {
-    super(appId, shuffleId, partitionId, readBufferSize, expectBlockIds, processBlockIds);
+      Configuration conf,
+      ShuffleDataDistributionType distributionType,
+      Roaring64NavigableMap expectTaskIds) throws Exception {
+    super(appId, shuffleId, partitionId, readBufferSize, expectBlockIds, processBlockIds,
+        distributionType, expectTaskIds);
     this.filePrefix = filePrefix;
     this.indexReader = createHdfsReader(ShuffleStorageUtils.generateIndexFileName(filePrefix), conf);
     this.dataReader = createHdfsReader(ShuffleStorageUtils.generateDataFileName(filePrefix), conf);
   }
 
+  // Only for test
+  public HdfsShuffleReadHandler(
+      String appId,
+      int shuffleId,
+      int partitionId,
+      String filePrefix,
+      int readBufferSize,
+      Roaring64NavigableMap expectBlockIds,
+      Roaring64NavigableMap processBlockIds,
+      Configuration conf) throws Exception {
+    this(appId, shuffleId, partitionId, filePrefix, readBufferSize, expectBlockIds,
+        processBlockIds, conf, ShuffleDataDistributionType.NORMAL, Roaring64NavigableMap.bitmapOf());
+  }
+
   @Override
   protected ShuffleIndexResult readShuffleIndex() {
     long start = System.currentTimeMillis();
     try {
       byte[] indexData = indexReader.read();
-      int segmentNumber = (int) (indexData.length / FileBasedShuffleSegment.SEGMENT_SIZE);
+      int segmentNumber = indexData.length / FileBasedShuffleSegment.SEGMENT_SIZE;
       int expectedLen = segmentNumber * FileBasedShuffleSegment.SEGMENT_SIZE;
       if (indexData.length != expectedLen) {
         LOG.warn("Maybe the index file: {} is being written due to the shuffle-buffer flushing.", filePrefix);
