@@ -102,7 +102,7 @@ public class HdfsShuffleWriteHandler implements ShuffleWriteHandler {
 
   @Override
   public void write(
-      List<ShufflePartitionedBlock> shuffleBlocks) throws IOException, IllegalStateException {
+      List<ShufflePartitionedBlock> shuffleBlocks) throws Exception {
     final long start = System.currentTimeMillis();
     writeLock.lock();
     try {
@@ -127,7 +127,7 @@ public class HdfsShuffleWriteHandler implements ShuffleWriteHandler {
             "Write handler inside cost {} ms for {}",
             (System.currentTimeMillis() - ss),
             fileNamePrefix);
-      } catch (Exception e) {
+      } catch (IOException e) {
         LOG.warn("Write failed with " + shuffleBlocks.size() + " blocks for " + fileNamePrefix + "_" + failTimes, e);
         failTimes++;
         throw new RuntimeException(e);
