@@ -19,32 +19,48 @@ package org.apache.uniffle.storage.common;
 
 import java.io.File;
 import java.io.IOException;
+import java.nio.file.Files;
+import java.nio.file.attribute.PosixFilePermission;
 import java.util.List;
+import java.util.Set;
 
 import com.google.common.collect.Lists;
+import com.google.common.collect.Sets;
 import org.junit.jupiter.api.AfterAll;
 import org.junit.jupiter.api.BeforeAll;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.io.TempDir;
 import org.roaringbitmap.RoaringBitmap;
 
+import org.apache.uniffle.common.storage.StorageMedia;
 import org.apache.uniffle.common.util.RssUtils;
 import org.apache.uniffle.storage.request.CreateShuffleWriteHandlerRequest;
 import org.apache.uniffle.storage.util.StorageType;
 
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertNull;
+import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 
 
 public class LocalStorageTest {
 
   private static File testBaseDir;
+  private static File testBaseDirWithoutPermission;
+  private static String mountPoint;
 
   @BeforeAll
   public static void setUp(@TempDir File tempDir) throws IOException  {
     testBaseDir = new File(tempDir, "test");
+    testBaseDirWithoutPermission = new File(tempDir, "test-no-permission");
     testBaseDir.mkdir();
+    testBaseDirWithoutPermission.mkdirs();
+    try {
+      mountPoint = Files.getFileStore(testBaseDir.toPath()).name();
+    } catch (IOException ioe) {
+      // pass
+    }
   }
 
   @AfterAll
@@ -52,15 +68,19 @@ public class LocalStorageTest {
     testBaseDir.delete();
   }
 
-  @Test
-  public void canWriteTest() {
-    LocalStorage item = LocalStorage.newBuilder().basePath(testBaseDir.getAbsolutePath())
+  private LocalStorage createTestStorage(File baseDir) {
+    return LocalStorage.newBuilder().basePath(baseDir.getAbsolutePath())
         .cleanupThreshold(50)
         .highWaterMarkOfWrite(95)
         .lowWaterMarkOfWrite(80)
         .capacity(100)
         .cleanIntervalMs(5000)
         .build();
+  }
+
+  @Test
+  public void canWriteTest() {
+    LocalStorage item = createTestStorage(testBaseDir);
 
     item.getMetaData().updateDiskSize(20);
     assertTrue(item.canWrite());
@@ -74,6 +94,33 @@ public class LocalStorageTest {
     assertTrue(item.canWrite());
   }
 
+  @Test
+  public void baseDirectoryInitTest() throws IOException {
+    // empty and writable base dir
+    File newBaseDir = new File(testBaseDirWithoutPermission, "test-new");
+    assertTrue(newBaseDir.mkdirs());
+    // then remove write permission to parent dir.
+    Set<PosixFilePermission> perms = Sets.newHashSet();
+    perms.add(PosixFilePermission.OWNER_READ);
+    perms.add(PosixFilePermission.OWNER_EXECUTE);
+    Files.setPosixFilePermissions(testBaseDirWithoutPermission.toPath(), perms);
+    createTestStorage(newBaseDir);
+    // non-empty and writable base dir
+    File childDir = new File(newBaseDir, "placeholder");
+    assertTrue(childDir.mkdirs());
+    createTestStorage(newBaseDir);
+    // base dir is configured to a wrong file instead of dir
+    File emptyFile = new File(newBaseDir, "empty_file");
+    assertTrue(emptyFile.createNewFile());
+    assertThrows(RuntimeException.class, () -> {
+      createTestStorage(emptyFile);
+    });
+
+    // not existed base dir should work
+    File notExisted = new File(newBaseDir, "not_existed");
+    createTestStorage(notExisted);
+  }
+
   @Test
   public void removeResourcesTest() throws Exception {
     LocalStorage item = prepareDiskItem();
@@ -87,13 +134,7 @@ public class LocalStorageTest {
   }
 
   private LocalStorage prepareDiskItem() {
-    final LocalStorage item = LocalStorage.newBuilder().basePath(testBaseDir.getAbsolutePath())
-        .cleanupThreshold(50)
-        .highWaterMarkOfWrite(95)
-        .lowWaterMarkOfWrite(80)
-        .capacity(100)
-        .cleanIntervalMs(5000)
-        .build();
+    final LocalStorage item = createTestStorage(testBaseDir);
     RoaringBitmap partitionBitMap = RoaringBitmap.bitmapOf();
     partitionBitMap.add(1);
     partitionBitMap.add(2);
@@ -131,13 +172,7 @@ public class LocalStorageTest {
 
   @Test
   public void diskMetaTest() {
-    LocalStorage item = LocalStorage.newBuilder().basePath(testBaseDir.getAbsolutePath())
-        .cleanupThreshold(50)
-        .highWaterMarkOfWrite(95)
-        .lowWaterMarkOfWrite(80)
-        .capacity(100)
-        .cleanIntervalMs(5000)
-        .build();
+    LocalStorage item = createTestStorage(testBaseDir);
     List<Integer> partitionList1 = Lists.newArrayList(1, 2, 3, 4, 5);
     List<Integer> partitionList2 = Lists.newArrayList(6, 7, 8, 9, 10);
     List<Integer> partitionList3 = Lists.newArrayList(1, 2, 3);
@@ -164,6 +199,31 @@ public class LocalStorageTest {
     assertEquals(2, item.getSortedShuffleKeys(false, 3).size());
   }
 
+  @Test
+  public void diskStorageInfoTest() {
+    LocalStorage item = LocalStorage.newBuilder()
+        .basePath(testBaseDir.getAbsolutePath())
+        .cleanupThreshold(50)
+        .highWaterMarkOfWrite(95)
+        .lowWaterMarkOfWrite(80)
+        .capacity(100)
+        .cleanIntervalMs(5000)
+        .build();
+    assertEquals(mountPoint, item.getMountPoint());
+    assertNull(item.getStorageMedia());
+
+    LocalStorage itemWithStorageType = LocalStorage.newBuilder()
+        .basePath(testBaseDir.getAbsolutePath())
+        .cleanupThreshold(50)
+        .highWaterMarkOfWrite(95)
+        .lowWaterMarkOfWrite(80)
+        .capacity(100)
+        .cleanIntervalMs(5000)
+        .localStorageMedia(StorageMedia.SSD)
+        .build();
+    assertEquals(StorageMedia.SSD, itemWithStorageType.getStorageMedia());
+  }
+
   @Test
   public void writeHandlerTest() {
     LocalStorage item = LocalStorage.newBuilder().basePath(testBaseDir.getAbsolutePath()).build();
